import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import _CloneDeep from 'lodash/cloneDeep';
import store from 'stores/LocationStore';
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { positionOffset, convertMillimetersToMeters } from 'utils/formatters';
import { macToUpperCase, userHasRole, getAnchorType, getTrackableType, getDeviceTypeClass, isAnchorLive, getTrackableStatus, hasAlarms, isPerson, isVehicle, isAssetTag } from 'utils/utils';
import { getKeyValueHtml, getKeyValuesHtml, getNodeBatteryLevel, getLeftCornerRibbonHtml } from 'utils/htmlUtils';
import { getLastReported } from 'utils/timeUtils';
import { TRACKABLE_POP_UP, DB_NAMES, COMPONENTS } from 'common/ui-labels';
import { USER_ROLES } from 'common/user-roles';
import { TRACKABLE_CATEGORY } from 'common/constants';
import { PopUpWrapper } from 'components/leaflet/PopUpWrapper';


export const trackablePopUp = (trackable, sublocation, onClose = () => { }, playbackSpeed, intl) => {
    const user = store.getCurrentUser();
    const { isOpenPlayBackControl } = store.getPlayback();
    const isUserProjectSystemAdmin = user && user.access &&
        (userHasRole(user.access, USER_ROLES.PROJECT_SYSTEM_ADMIN));

    if (isUserProjectSystemAdmin && trackable && trackable.uid && trackable.position && sublocation) {
        const trackablePosition = positionOffset(trackable.position, sublocation);
        if (!trackablePosition) {
            return null;
        }

        const { BATTERY_FUNC } = COMPONENTS.FIELDS.NODE;

        const info = {
            [BATTERY_FUNC(intl)]: getNodeBatteryLevel(trackable.node, isAssetTag(trackable.category)),
            [TRACKABLE_POP_UP.TAG_MAC(intl)]: macToUpperCase(_Get(trackable, 'node.mac_address')),
            [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(_Get(trackable, 'node.last_heard'), intl),
        };

        const trackableGroupsString = trackable.groupList.join(', ');
        if (trackableGroupsString.length > 0) {
            info[TRACKABLE_POP_UP.GROUPS(intl)] = trackableGroupsString;
        }

        const popupClass = ['RPP-BlockBody', `Leaflet-Transition-Speed${playbackSpeed}`];
        const { alarms } = trackable;
        if (alarms && alarms.length > 0) {
            popupClass.push('alarming');
            info[TRACKABLE_POP_UP.CURRENT_ALARMS(intl)] = alarms.map(alarm => alarm.additional_info.text);
        }

        const cornerRibbonTextLabel = getTrackableType(intl, trackable);

        return (
            <PopUpWrapper
                position={trackablePosition}
                closeButton={false}
                onClose={onClose}
                className={popupClass.join(' ')}
            >
                <div className="RPP-BlockBody-Wrapper">
                    <div className="RPP-BlockHeader">
                        {getLeftCornerRibbonHtml(cornerRibbonTextLabel, getDeviceTypeClass(trackable))}
                        <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                            { trackable.name }
                        </h4>
                        {!trackable.isResponsive(isOpenPlayBackControl) && <div className="unresponsive-message text-danger">
                            {TRACKABLE_POP_UP.TRACKABLE_NOT_RESPONDING(intl)}!
                        </div>}
                        <button
                            type="button"
                            className="RPP-BtnPopUp"
                            onClick={onClose}
                        >
                            <i className="fa fa-times" />
                        </button>
                    </div>
                    <div className="RPP-PopUpRowsWrapper">
                        {Object.keys(info).map((key, i) => {
                            if (key === TRACKABLE_POP_UP.CURRENT_ALARMS(intl)) {
                                if (info[key]) {
                                    return getKeyValuesHtml(key, info[key], i);
                                }
                                return <div key={i} />;
                            }
                            return getKeyValueHtml(key, info[key], i);
                        })}
                    </div>
                </div>
            </PopUpWrapper>
        );
    }
    return null;
};

/**
 * New popup with leaflet pop up
 * @param _anchor
 * @param sublocation
 * @param onClose
 * @param intl
 * @returns {*}
 */
export const anchorPopUp = (_anchor, sublocation, onClose = () => { }, intl) => {
    const anchor = _CloneDeep(_anchor);
    const user = store.getCurrentUser();
    const isUserProjectSystemAdmin = user && user.access
        && (userHasRole(user.access, USER_ROLES.PROJECT_SYSTEM_ADMIN));
    const node = store.getNodeByMacAddress(_Get(anchor, 'node.mac_address', false));

    if (node) {
        anchor.node.last_heard = node.last_heard;
    }

    if (isUserProjectSystemAdmin && anchor && anchor.uid && sublocation) {
        const anchorPosition = positionOffset(anchor.position, sublocation);
        if (!anchorPosition) {
            return null;
        }

        const info = {
            [TRACKABLE_POP_UP.TAG_MAC(intl)]: macToUpperCase(_Get(anchor, 'node.mac_address') || ''),
            [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(_Get(anchor, 'node.last_heard', null), intl),
        };

        const isAlive = isAnchorLive(anchor);

        const cornerRibbonTextLabel = getAnchorType(intl, anchor);

        const popupClass = ['RPP-BlockBody'];

        return (
            <PopUpWrapper
                position={anchorPosition}
                closeButton={false}
                onClose={onClose}
                className={popupClass.join(' ')}
            >
                <div className="RPP-BlockBody-Wrapper" onClick={onClose}>
                    <div className="RPP-BlockHeader">
                        {getLeftCornerRibbonHtml(cornerRibbonTextLabel, getDeviceTypeClass(anchor))}
                        <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                            {anchor.name}
                        </h4>
                        {!isAlive && <div className="unresponsive-message text-danger">
                            {TRACKABLE_POP_UP.NOT_RESPONDING(intl)}!
                        </div>}
                        <button
                            type="button"
                            className="RPP-BtnPopUp"
                            onClick={onClose}
                        >
                            <i className="fa fa-times" />
                        </button>
                    </div>
                    <div className="RPP-PopUpRowsWrapper">
                        {Object.keys(info).map((key, i) => getKeyValueHtml(key, info[key], i))}
                    </div>
                </div>
            </PopUpWrapper>
        );
    }
    return null;
};

const getDriverVehicle = (trackables) => {
    const driverTko = trackables.find(tko => isPerson(tko.category)) || trackables[0];
    const vehicleTko = trackables.find(tko => isVehicle(tko.category)) || trackables[1];

    if (driverTko && vehicleTko) {
        return { driverTko, vehicleTko };
    }

    return { driverTko: trackables[0], vehicleTko: trackables[1] };
};

export const vdaPopUp = (selectedClusterTrackables, sublocation, onClose = () => { }, playbackSpeed, intl) => {
    const { driverTko, vehicleTko } = getDriverVehicle(selectedClusterTrackables);

    const { BATTERY_FUNC, GROUPS_FUNC } = COMPONENTS.FIELDS.NODE;

    if (vehicleTko && driverTko) {
        const driverGroups = driverTko.groupList.join(', ');
        const vehicleGroups = vehicleTko.groupList.join(', ');

        const info = {
            [BATTERY_FUNC(intl)]: getNodeBatteryLevel(driverTko.node),
            ...(driverGroups && { [GROUPS_FUNC(intl)]: driverGroups }),
            [DB_NAMES.VEHICLE(intl)]: vehicleTko.name,
            ...(vehicleGroups && { [TRACKABLE_POP_UP.VEHICLE_GROUP(intl)]: vehicleGroups }),
        };

        const vehicleAlarms = vehicleTko.alarms;
        const driverAlarms = driverTko.alarms;
        const isVehicleAlarmed = vehicleAlarms && vehicleAlarms.length > 0;
        const isDriverAlarmed = driverAlarms && driverAlarms.length > 0;
        const isVDAAlarmed = isDriverAlarmed || isVehicleAlarmed;

        const popupClass = ['RPP-BlockBody', `Leaflet-Transition-Speed${playbackSpeed}`];
        const popupPosition = positionOffset(vehicleTko.position, sublocation);

        if (isVDAAlarmed) {
            popupClass.push('alarming');
            if (isVehicleAlarmed) {
                info[TRACKABLE_POP_UP.VDA.VEHICLE_ALARMS(intl)] = vehicleAlarms.map(alarm => alarm.additional_info.text);
            }
            if (isDriverAlarmed) {
                info[TRACKABLE_POP_UP.VDA.DRIVER_ALARMS(intl)] = driverAlarms.map(alarm => alarm.additional_info.text);
            }
        }

        return (
            <PopUpWrapper
                position={popupPosition}
                closeButton={false}
                onClose={onClose}
                className={popupClass.join(' ')}
            >
                <div className="RPP-BlockBody-Wrapper">
                    <div className="RPP-BlockHeader">
                        {getLeftCornerRibbonHtml(TRACKABLE_POP_UP.DRIVER(intl), getDeviceTypeClass(driverTko))}
                        <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                            { driverTko.name }
                        </h4>
                        <button
                            type="button"
                            className="RPP-BtnPopUp"
                            onClick={onClose}
                        >
                            <i className="fa fa-times" />
                        </button>
                    </div>
                    <div className="RPP-PopUpRowsWrapper">
                        {Object.keys(info).map((key, i) => {
                            if (key === TRACKABLE_POP_UP.VDA.DRIVER_ALARMS(intl) || key === TRACKABLE_POP_UP.VDA.VEHICLE_ALARMS(intl)) {
                                if (info[key]) {
                                    return getKeyValuesHtml(key, info[key], i);
                                }
                                return <div key={i} />;
                            }
                            return getKeyValueHtml(key, info[key], i);
                        })}
                    </div>
                </div>
            </PopUpWrapper>
        );
    }
    return null;
};

/**
 *
 * @param trackablesCluster
 * @param trackablesCluster.unresponsiveTrackablesCount
 * @param trackablesCluster.trackableUids
 * @param trackablesCluster.selectedClusterTrackables
 * @param popupPosition
 * @param onClosePopup
 * @param intl
 * @return {*}
 */
export const clusterPopUpNew = (trackablesCluster, popupPosition, onClosePopup, intl) => {
    if (trackablesCluster && popupPosition) {
        const popupClass = ['RPP-BlockBody trackable-cluster-popup'];

        const { unresponsiveTrackablesCount, selectedClusterTrackables } = trackablesCluster;

        const getTableData = selectedClusterTrackables.map((tko, i) => ({
            id: i,
            uid: tko.uid,
            trackableName: tko.name,
            trackableStatus: getTrackableStatus(tko, intl),
            isResponsive: tko.isResponsive(),
            hasAlarm: hasAlarms(tko),
        }));

        const createCustomToolBar = props => (<div className="col-md-12">{ props.components.searchPanel }</div>);

        const options = {
            defaultSortName: 'trackableName',
            toolBar: createCustomToolBar,
        };

        const expandComponent = (row) => {
            const expandedTrackable = selectedClusterTrackables.find(to => (to.uid === row.uid));

            if (expandedTrackable && expandedTrackable.node) {
                const { node } = expandedTrackable;

                const info = {
                    [TRACKABLE_POP_UP.BATTERY_LEVEL(intl)]: getNodeBatteryLevel(node, isAssetTag(expandedTrackable.category)),
                    [TRACKABLE_POP_UP.TAG_MAC(intl)]: macToUpperCase(node.mac_address),
                    [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(node.last_heard, intl),
                };

                const trackableGroupsString = expandedTrackable.groupList.join(', ');
                if (trackableGroupsString.length > 0) {
                    info[TRACKABLE_POP_UP.GROUPS(intl)] = trackableGroupsString;
                }

                if (hasAlarms(expandedTrackable)) {
                    info[TRACKABLE_POP_UP.CURRENT_ALARMS(intl)] = expandedTrackable.alarms.map(to => to.additional_info.text);
                }

                return (<div className="RPP-PopUpRowsWrapper expanded-rows">
                    {Object.keys(info).map((key, i) => {
                        if (key === TRACKABLE_POP_UP.CURRENT_ALARMS(intl)) {
                            if (info[key]) {
                                return getKeyValuesHtml(key, info[key], i);
                            }
                            return <div key={i} />;
                        }
                        return getKeyValueHtml(key, info[key], i);
                    })}
                </div>);
            }
            return '';
        };

        /**
         *
         * @param props.isExpandableRow
         * @param props.isExpanded
         * @returns {JSX.Element}
         * check prev commit on UI
         */
        const expandColumnComponent = props => (
            <div>
                <i className={`fa fa-angle-${props.isExpanded ? 'up' : 'down'}`} />
            </div>
        );

        const columnClassNameFormat = (fieldValue, row) => {
            const { isResponsive, hasAlarm } = row;
            const responsiveState = isResponsive ? 'success' : 'danger';
            return `padding-l-m text-${hasAlarm ? 'alarmed' : responsiveState}`;
        };

        return (
            <PopUpWrapper
                position={popupPosition}
                closeButton={false}
                onClose={onClosePopup}
                className={popupClass.join(' ')}
            >
                <div className="RPP-BlockBody-Wrapper">
                    <div className="RPP-BlockHeader">
                        {getLeftCornerRibbonHtml('Cluster', getDeviceTypeClass(trackablesCluster))}
                        <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                            <strong>{selectedClusterTrackables.length}</strong> {TRACKABLE_POP_UP.TRACKABLE_OR_TRACKABLES_FUNC(intl)}
                        </h4>
                        {unresponsiveTrackablesCount > 0 && <div className="unresponsive-message text-danger">
                            <strong>{unresponsiveTrackablesCount}</strong> {TRACKABLE_POP_UP.NOT_RESPONDING(intl)}!
                        </div>}
                        <button
                            type="button"
                            className="RPP-BtnPopUp"
                            onClick={onClosePopup}
                        >
                            <i className="fa fa-times" />
                        </button>
                    </div>
                    <div className="RPP-PopUpRowsWrapper">
                        <BootstrapTable
                            data={getTableData}
                            options={options}
                            tableHeaderClass="hidden"
                            tableBodyClass="lineless-table popup-info"
                            trClassName="cursor-pointer"
                            bordered={false}
                            expandComponent={expandComponent}
                            expandableRow={() => true}
                            expandColumnOptions={{
                                expandColumnVisible: true,
                                expandColumnComponent,
                                columnWidth: 20,
                            }}
                            search
                            hover
                        >
                            <TableHeaderColumn
                                dataField="id"
                                searchable={false}
                                hidden
                                isKey
                            />
                            <TableHeaderColumn
                                columnClassName="text-ellipsis"
                                dataField="trackableName"
                                tdAttr={{ 'data-id': 'trackableName' }}
                            />
                            <TableHeaderColumn
                                columnClassName={columnClassNameFormat}
                                dataField="trackableStatus"
                                tdAttr={{ 'data-id': 'trackableStatus' }}
                            />
                        </BootstrapTable>
                    </div>
                </div>
            </PopUpWrapper>
        );
    }
    return null;
};


export function VertexPopup(props) {
    if (!props.show) {
        return (<Fragment />);
    }
    return (
        <PopUpWrapper
            position={props.position}
            onClose={() => {
                props.onClose();
            }}
            className="RPP-BlockBody-Small"
        >
            <div className="RPP-BlockBody-Wrapper" style={{ textAlign: 'center' }}>
                <div className="RPP-BlockHeader">
                    <button
                        type="button"
                        className="RPP-BtnPopUp"
                        onClick={() => { props.onClose(); }}
                    >
                        <i className="fa fa-times" />
                    </button>
                </div>
                <div className="RPP-BlockHeader-Bold">Point: {props.index}</div>
                <div className="RPP-PopUpRowsWrapper">
                    <span>X: {convertMillimetersToMeters(props.data.x)} m</span>
                    <span style={{ marginLeft: '5px', marginRight: '5px' }}>|</span>
                    <span>Y: {convertMillimetersToMeters(props.data.y)} m</span>
                </div>
            </div>
        </PopUpWrapper>);
}

VertexPopup.propTypes = {
    position: PropTypes.shape({}),
    onClose: PropTypes.func,
    index: PropTypes.number,
    show: PropTypes.bool,
    data: PropTypes.shape({
        x: PropTypes.number,
        y: PropTypes.number,
    }),
};

VertexPopup.defaultProps = {
    position: {},
    onClose: () => {},
    show: false,
    index: 0,
    data: {
        x: 0,
        y: 0,
    },
};


export function NearMissPopUp(props) {
    const { nearMissMarker, onClosePopup } = props;
    
    if (!nearMissMarker) {
        return null;
    }
    
    return (
        <PopUpWrapper
            className="near-miss-popup"
            position={nearMissMarker.position}
            onClose={() => { onClosePopup(); }}
        >
            <div>
                <div className="RPP-PopUpRowsWrapper">
                    <div className="popup-row row">
                        <div className="col-md-4 popup-field-label">Distance (m):</div>
                        <div className="col-md-8 popup-field-value">
                            {convertMillimetersToMeters(nearMissMarker.distance).toFixed(1)}
                        </div>
                    </div>
                    <div className="popup-row row">
                        <div className="col-md-4 popup-field-label">Trigger time:</div>
                        <div className="col-md-8 popup-field-value">
                            {nearMissMarker.time}
                        </div>
                    </div>
                    {nearMissMarker.trackables.map(trackable => (
                        <div key={trackable.uid}>
                            <div className="popup-row row">
                                <div className="col-md-4 popup-field-label">{trackable.category === TRACKABLE_CATEGORY.VEHICLE ? 'Vehicle:' : 'Person:'}</div>
                                <div className="col-md-8 popup-field-value">
                                    {trackable.name}
                                </div>
                            </div>
                            {trackable.driver_uid && <div className="popup-row row">
                                <div className="col-md-4 popup-field-label">Driver:</div>
                                <div className="col-md-8 popup-field-value">{trackable.driver_name || ''}</div>
                            </div>}
                            <div className="popup-row row">
                                <div className="col-md-4 popup-field-label">Coordinate (m):</div>
                                <div className="col-md-8 popup-field-value">
                                    X: {convertMillimetersToMeters(trackable.x).toFixed(1)}, Y: {convertMillimetersToMeters(trackable.y).toFixed(1)}
                                </div>
                            </div>
                        </div>))
                    }
                </div>
            </div>
        </PopUpWrapper>);
}

NearMissPopUp.defaultProps = {
    nearMissMarker: {},
    onClosePopup: () => {},
};

NearMissPopUp.propTypes = {
    nearMissMarker: PropTypes.shape({}),
    onClosePopup: PropTypes.func,
};
