import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { change, Field, formValueSelector, reduxForm } from 'redux-form';
import 'react-select/dist/react-select.css';
import { COMPONENTS, LABELS, TRACKABLE_MODAL, NAMES_OF_TABS } from 'common/ui-labels';
import { NAME_OF_FORMS, DEFAULT_VALUES, ERROR_TEXT, POS_UPDATE_INTERVAL } from 'common/constants';
import RenderTooltipHelper from 'components/RenderTooltipHelper';
import { getLength, isAssetTag, sortByField } from 'utils/utils';
import { validatePWS } from 'utils/pwsConfigUtils';
import SelectComponent from 'components/modals/SelectComponent';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import LowLatencyLinkComponent from 'components/modals/LowLatencyLink';
import CustomInfoFieldComponent from 'components/modals/CustomInfoField';
import Trackable from 'stores/classes/Trackable';
import Group from 'stores/classes/Group';
import { ALL_TYPE_VALUES, CATEGORIES, isSelectedFields } from '../TrackableEditModal/trackableModalUtils';
import renderBufferSection from '../TrackableEditModal/bufferSection';
import advancedOptionsSection from '../TrackableEditModal/advancedOptionsSection';
import Buttons from './Buttons';


const validate = (values, props) => {
    const { intl } = props;
    const errors = {};

    if (!Array.isArray(values.editedTrackables) || !values.editedTrackables.length) {
        errors.editedTrackables = COMPONENTS.VALIDATION.REQUIRED;
    }
    
    if (!values.category) {
        errors.category = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (values.buffer_radius <= 0) {
        errors.buffer_radius = TRACKABLE_MODAL.EDIT.FORM.ERROR_MUST_BE_POSITIVE;
    }
    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }
    
    if ((values.position_update_interval < POS_UPDATE_INTERVAL.MIN || values.position_update_interval > POS_UPDATE_INTERVAL.MAX) && isAssetTag(values.category)) {
        errors.position_update_interval = TRACKABLE_MODAL.EDIT.FORM.ERROR_WRONG_UPDATE_INTERVAL(POS_UPDATE_INTERVAL.MIN, POS_UPDATE_INTERVAL.MAX / (60 * 60));
    }
    
    validatePWS(values, errors, intl);
    
    return errors;
};

let TrackableBulkEditForm = (props) => {
    const {
        close, submit, groups, intl, posReportInterval, dispatch, trackableCategory, invalid, editedTrackables,
        isBufferSelected, isStationary, role, angle, trackables, t2tNoAlarmUid, customInfoLength, submitted,
        safe_distance, distance, alert_distance, delay, categoryOptions, isAdvancedOptions, className, pin,
    } = props;
    const editedUids = (editedTrackables || []).map(tko => tko.value);
    const noAlarmOptions = trackables.filter(tko => !editedUids.includes(tko.uid));
    
    const formDispatchChange = (field, value) => {
        dispatch(change(NAME_OF_FORMS.TRACKABLE_BULK_EDIT, field, value));
    };
    const isCategoryAsset = isAssetTag(trackableCategory);
    const clearRoleField = () => formDispatchChange('role', '');

    return (
        <form className={className}>
            <div className="row">
                <div
                    className="col-md-12"
                    style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight: '60vh' }}
                >
                    <Fragment key="editedTrackables">
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4">{NAMES_OF_TABS.TRACKABLE_OBJECTS}*</div>
                            <div htmlFor="editedTrackables" className="col-md-8" data-id="trackable-edit-group-list">
                                <Field
                                    name="editedTrackables"
                                    type="text"
                                    label="Objective"
                                    options={trackables}
                                    placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                    noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                                    component={MultiSelectComponent}
                                />
                            </div>
                        </div>
                    </Fragment>
                    <Fragment key="category">
                        <div className="Form-Line row">
                            <div htmlFor="category" className="Form-Label col-md-4">
                                {TRACKABLE_MODAL.EDIT.FORM.TRACKABLE_CATEGORY_FUNC(intl)}*
                                <RenderTooltipHelper
                                    tooltipText={TRACKABLE_MODAL.EDIT.FORM.TRACKABLE_CATEGORY_HELP_FUNC(intl)}
                                    placement="bottom"
                                />
                            </div>
                            <div className="col-md-8" data-id="trackable-edit-categories">
                                <Field
                                    onChange={
                                        (e, newVal) => {
                                            if (trackableCategory !== newVal) {
                                                clearRoleField();
                                            }
                                        }
                                    }
                                    clearable={false}
                                    className="Form-Input"
                                    name="category"
                                    component={SelectComponent}
                                    simpleValue
                                    placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                    noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                                    options={categoryOptions}
                                />
                            </div>
                        </div>
                    </Fragment>

                    <hr />
            
                    {renderBufferSection({ dispatch, isBufferSelected, uid: null, intl, trackableCategory })}
            
                    <hr />
            
                    {trackableCategory && advancedOptionsSection({
                        dispatch,
                        isAdvancedOptions,
                        posReportInterval,
                        intl,
                        isCategoryAsset,
                        uid: null,
                        hideStationary: true,
                        trackableCategory,
                        clearRoleField,
                    })}
                    <hr />
                    <LowLatencyLinkComponent
                        safe_distance={safe_distance}
                        distance={distance}
                        alert_distance={alert_distance}
                        delay={delay}
                        angle={angle}
                        isAdvanced={isAdvancedOptions}
                        dispatch={dispatch}
                        intl={intl}
                        role={role}
                        t2tNoAlarmUid={t2tNoAlarmUid}
                        trackables={noAlarmOptions}
                        showAlarmDisableForField
                        groups={groups}
                        formDispatchChange={formDispatchChange}
                        trackableCategory={trackableCategory}
                        isStationary={isStationary}
                        pin={pin}
                    />
                    <CustomInfoFieldComponent
                        placeholder={LABELS.CUSTOM_INFO_PLACEHOLDER(intl)}
                        dataId="trackable-edit-custom-info"
                        customInfoLength={customInfoLength}
                    />
                </div>
            </div>
            
            <Buttons
                close={close}
                submit={submit}
                submitted={submitted}
                invalid={invalid}
            />
        </form>
    );
};

TrackableBulkEditForm.propTypes = {
    dispatch: PropTypes.func,
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    submitted: PropTypes.bool,
    groups: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string, // x.uid
        label: PropTypes.string, // x.name
    })),
    editedTrackables: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string, // x.uid
        label: PropTypes.string, // x.name
    })),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    posReportInterval: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.number,
        label: PropTypes.string,
        type: PropTypes.string,
    })),
    trackableCategory: PropTypes.string,
    isBufferSelected: PropTypes.bool,
    isAdvancedOptions: PropTypes.bool,
    isStationary: PropTypes.bool,
    role: PropTypes.string,
    angle: PropTypes.number,
    safe_distance: PropTypes.number,
    distance: PropTypes.number,
    alert_distance: PropTypes.number,
    delay: PropTypes.number,
    trackables: PropTypes.arrayOf(PropTypes.shape()),
    t2tNoAlarmUid: PropTypes.string,
    customInfoLength: PropTypes.number,
    categoryOptions: PropTypes.arrayOf(PropTypes.shape()),
    pin: PropTypes.string,
};

TrackableBulkEditForm.defaultProps = {
    dispatch: () => {},
    close: () => {},
    submit: () => {},
    className: '',
    invalid: true,
    submitted: false,
    groups: [],
    editedTrackables: [],
    intl: { formatMessage: () => {} },
    posReportInterval: [],
    trackableCategory: '',
    isBufferSelected: false,
    isAdvancedOptions: false,
    isStationary: false,
    role: '',
    angle: DEFAULT_VALUES.LLL_COVERAGE_ANGLE,
    safe_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
    distance: DEFAULT_VALUES.LLL_COVERAGE_DISTANCE,
    alert_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
    delay: DEFAULT_VALUES.LLL_DELAY,
    trackables: [],
    t2tNoAlarmUid: '',
    customInfoLength: 0,
    categoryOptions: [],
    pin: '',
};

TrackableBulkEditForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_BULK_EDIT,
    validate,
})(TrackableBulkEditForm);


const getCategoryOptions = (categories, isBufferEnabled) => categories.map((cat) => {
    if (isAssetTag(cat.value)) {
        return { ...cat, disabled: isBufferEnabled };
    }
    return { ...cat };
});

const mapStateToForm = (state) => {
    const bulkEdit = state.modals.trackableModals.trackableBulkEdit;
    const initialValues = {
        editedTrackables: bulkEdit.editedTrackables.map(tko => ({ value: tko.uid, label: tko.name })),
        isStationary: false,
        defaultHeight: DEFAULT_VALUES.trackable_default_height,
        position_update_interval: DEFAULT_VALUES.trackable_update_interval,
        buffer_radius: DEFAULT_VALUES.trackable_buffer_radius,
    };
    const selector = formValueSelector(NAME_OF_FORMS.TRACKABLE_BULK_EDIT);
    const trackableCategory = selector(state, 'category');

    const selectedFieldsObject = selector(
        state,
        'buffer', 'buffer_radius', 'active_buffer', 'buffer_exceptions', 'zones', 'stationary', 'advanced_options',
        'tag_config', 'role', 'angle', 't2t_no_alarm_uid', 'custom_info', 'editedTrackables',
        'safe_distance', 'distance', 'alert_distance', 'delay', 'safe_distance_prevent', 'pin',
    );
    const {
        role,
        safe_distance,
        distance,
        alert_distance,
        delay,
        angle,
        t2t_no_alarm_uid,
        custom_info,
        editedTrackables,
        pin,
    } = selectedFieldsObject;
    const { isBufferSelected, isAdvancedOptions } = isSelectedFields(initialValues.checkboxes || {}, selectedFieldsObject);
    initialValues.advanced_options = isAdvancedOptions;
    const groups = sortByField(Group.getAllGroupsArray(state), 'name').map(group => ({ value: group.uid, label: group.name }));
    const trackables = sortByField(Trackable.getAllTrackablesArray(state), 'name').map(tko => ({ ...tko, value: tko.uid, label: tko.name }));
    const categoryOptions = getCategoryOptions(CATEGORIES, isBufferSelected);

    return {
        submitted: bulkEdit.submitted,
        initialValues,
        groups,
        angle,
        trackables,
        editedTrackables,
        trackableCategory,
        t2tNoAlarmUid: t2t_no_alarm_uid,
        safe_distance,
        distance,
        alert_distance,
        delay,
        role,
        isBufferSelected,
        isAdvancedOptions,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        posReportInterval: ALL_TYPE_VALUES,
        categoryOptions,
        pin,
    };
};

TrackableBulkEditForm = connect(mapStateToForm)(injectIntl(TrackableBulkEditForm));

export default TrackableBulkEditForm;
