import React from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import PropTypes from 'prop-types';

import {
    openUserDetailsModal, openUserEditModal, openUserDeleteModal, editUser, changeSelectedItems, selectItems,
    unselectItems,
} from '../../stores/actions';
import { trOddEvenClassFormat } from '../../utils';
import {
    USER_ROLES,
    DEFAULT_VALUES,
    USERS_TAB,
    TABLES,
    COMPONENTS,
    BOOTSTRAP_TABLE_SORT,
    NAMES_OF_TABS,
} from '../../common';
import User from './../../stores/classes/User';
import InfoTable from '../../components/InfoTable';
import expandColumnComponent from './../../components/ExpandColumnComponent';

class UsersTab extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        users: PropTypes.arrayOf(PropTypes.shape({})),
        projects: PropTypes.arrayOf(PropTypes.shape({})),
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        currentUser: PropTypes.shape({
            email: PropTypes.string,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        selectedProject: PropTypes.shape({}),
        totalCount: PropTypes.number,
    };

    static defaultProps = {
        dispatch: () => {},
        users: [],
        projects: [],
        selectedItems: [],
        currentUser: {
            email: '',
        },
        selectedProject: {},
        totalCount: 0,
        intl: {
            formatMessage: () => {
            },
        },
    };

    setTypeToUser = (intl, cell, row) => {
        if (this.isProjectSelected()) {
            return (
                <select
                    className="form-control input-sm"
                    data-user-uid={row.uid}
                    onChange={e => this.updateUserType(e, row)}
                    defaultValue={row.userType}
                >
                    <option value={USER_ROLES.PROJECT_USER}>{USERS_TAB.USER_ROLES.PROJECT_USER(intl)}</option>
                    <option value={USER_ROLES.PROJECT_OPERATIONAL_ADMIN}>
                        {USERS_TAB.USER_ROLES.PROJECT_OPERATIONS_ADMIN(intl)}
                    </option>
                    <option
                        value={USER_ROLES.PROJECT_SYSTEM_ADMIN}
                    >{USERS_TAB.USER_ROLES.PROJECT_SYSTEM_ADMIN(intl)}
                    </option>
                </select>
            );
        } else if (row.userType === USER_ROLES.COMPANY_ADMIN || this.props.currentUser.email === row.email) { // company admin or current User
            return (
                <select className="form-control input-sm" defaultValue={row.userType} disabled>
                    <option value={USER_ROLES.COMPANY_ADMIN}>{USERS_TAB.USER_ROLES.COMPANY_ADMIN(intl)}</option>
                </select>
            );
        } else if (!row.userType) {
            // For removed users
            return (
                <select className="form-control input-sm" disabled />
            );
        }
        return (
            <select
                className="form-control input-sm"
                data-user-uid={row.uid}
                onChange={e => this.updateUserType(e, row)}
                value={row.userType}
            >
                <option value={USER_ROLES.PROJECT_USER}>{USERS_TAB.USER_ROLES.PROJECT_USER(intl)}</option>
                <option value={USER_ROLES.PROJECT_OPERATIONAL_ADMIN}>
                    {USERS_TAB.USER_ROLES.PROJECT_OPERATIONS_ADMIN(intl)}
                </option>
                <option
                    value={USER_ROLES.PROJECT_SYSTEM_ADMIN}
                >{USERS_TAB.USER_ROLES.PROJECT_SYSTEM_ADMIN(intl)}
                </option>
                <option value={USER_ROLES.COMPANY_ADMIN}>{USERS_TAB.USER_ROLES.COMPANY_ADMIN(intl)}</option>
            </select>
        );
    };

    getActionsIcons = (cell, row) => (
        <section>
            <i
                data-info="actions-section"
                data-id="users-actions-info"
                data-cy="user-table-info-button"
                onClick={this.showUserInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="users-actions-edit"
                data-cy="user-table-edit-button"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="users-actions-remove"
                data-cy="user-table-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    handleSelectAll = (isSelected) => {
        const { users } = this.props;
        if (isSelected) {
            this.props.dispatch(selectItems(users));
        } else {
            this.props.dispatch(unselectItems(users));
        }
    };

    showUserInfo = (row) => {
        this.props.dispatch(openUserDetailsModal(row));
    };

    openEditModal = (row) => {
        this.props.dispatch(openUserEditModal(row));
    };

    openAddUserModal = () => {
        this.props.dispatch(openUserEditModal({}));
    };

    openDeleteModal = (row) => {
        this.props.dispatch(openUserDeleteModal(row));
    };

    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const user = this.props.users.find(x => x.uid === item.uid);

            if (user) {
                params.push({
                    name: user.name,
                    uid: user.uid,
                    email: user.email,
                });
            }
        });

        if (params.length > 0) {
            this.props.dispatch(openUserDeleteModal(params));
        }
    };

    updateUserType = (e, row) => {
        row.userType = e.target.value;
        this.props.dispatch(editUser(row));
    };

    isProjectSelected = () => this.props.selectedProject;

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        return true;
    };

    expandComponent = (row) => {
        const rowInfo = {
            // Phone: row.user_details ? row.user_details.phone_number : '',
            [COMPONENTS.FIELDS.USER.PHONE_FUNC(this.props.intl)]: row.user_details ? row.user_details.phone_number : '',
        };
        return <InfoTable rowInfo={rowInfo} />;
    };

    isExpandableRow = () => true;

    createAddDeleteBtn = () => (
        <section>
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="users-button-trash"
                data-cy="user-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="users-button-plus"
                data-cy="user-table-add-button"
                onClick={this.openAddUserModal}
            />
        </section>);

    createCustomSearchField = intl => (
        <SearchField
            data-id="users-search"
            data-cy="user-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const options = {
            expandBy: 'column',
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
            columnWidth: '40px',
        };

        const { users } = this.props;
        return (
            <section className="AppLayout-Admin-Tab tab-projects-users">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.USERS_PROJECT_SCOPE}</h3>
                <h4 className="text-center AdminTab-Subtitle" data-id="users-total-count">
                    {USERS_TAB.TOTAL_COUNT_FUNC(this.props.totalCount, this.isProjectSelected() ? users.length : 0)}
                </h4>
                <hr />
                <BootstrapTable
                    condensed
                    data={users}
                    options={options}
                    hover
                    selectRow={selectRow}
                    expandableRow={this.isExpandableRow}
                    expandComponent={this.expandComponent}
                    trClassName={trOddEvenClassFormat}
                    tableBodyClass="automation-users-table"
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    expandColumnOptions={{
                        expandColumnVisible: true,
                        expandColumnComponent,
                        columnWidth: '40px',
                    }}
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        hidden
                        searchable={false}
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="name"
                        tdAttr={{ 'data-id': 'users-table-name' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="email"
                        tdAttr={{ 'data-id': 'users-table-email' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.EMAIL}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="project"
                        tdAttr={{ 'data-id': 'users-table-project' }}
                        dataAlign="center"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.PROJECT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="200px"
                        dataField="userType"
                        dataFormat={this.setTypeToUser.bind(this, this.props.intl)}
                        tdAttr={{ 'data-id': 'users-table-type' }}
                        dataAlign="center"
                        headerAlign="center"
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.USER.USER_TYPE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={this.getActionsIcons.bind(this)}
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const mapStateToProps = (state) => {
    const selectedProject = state.adminPage.projectSelect;
    const users = User.getAllUsers(state).reduce((memo, user) => {
        if (user.belongToProject(selectedProject)) {
            memo.push(user.getFormattedUser(selectedProject));
        }
        return memo;
    }, []);

    return {
        users,
        currentUser: state.user,
        projects: state.entities.projects,
        totalCount: User.getAllUsers(state).length,
        selectedItems: state.adminPage.selectedItems,
        selectedProject: state.adminPage.projectSelect,
    };
};

export default connect(mapStateToProps)(injectIntl(UsersTab));
