import * as api from 'api';

import { ACTIONS, MODAL_ACTIONS, STORAGE_KEY } from 'common/constants';
import { getPrivilegeFromUserType } from 'utils/utils';
import { isUserDefaultAdmin } from 'utils/simpleUtils';
import { USER_ROLES } from 'common/user-roles';
import { updateLabUserData } from 'stores/actions/userActions';
import { errorHandler, errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import * as storage from 'utils/browserStorage';


const filterUsersFromDefaultAdmin = users => users.filter(user => !isUserDefaultAdmin(user));

export const loadHelperLink = () => async (dispatch, getState) => {
    const { appContext } = getState();
    try {
        const helperLink = await api.getHelperLink();
        const hostName = storage._getHostName(appContext);
        dispatch({
            type: ACTIONS.HELPER_LINK_SET,
            params: hostName + helperLink,
        });
    } catch (err) {
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

export const loadMonitorLink = () => async (dispatch) => {
    try {
        const siteMonitorLink = await api.getSiteMonitorLink();
        dispatch({
            type: ACTIONS.SITE_MONITOR_LINK_SET,
            params: siteMonitorLink,
        });
    } catch (err) {
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

export const loadUsers = () => dispatch => new Promise((resolve) => {
    api.getUsers()
        .then((users) => {
            dispatch({
                type: ACTIONS.USERS_SET,
                params: filterUsersFromDefaultAdmin(users),
            });

            resolve();
        })
        .catch((err) => {
            resolve();
            console.warn(err);
            errorHandlerGetRequest(err, dispatch);
        });
});

function updateUserAccessInProject(args) {
    const {
        companyUid,
        projectUid,
        userUid,
        privilege,
        selectedProject,
        dispatch,
    } = args;

    if (!companyUid || !projectUid) return Promise.resolve(1);

    return api.updateUserAccess(companyUid, projectUid, userUid, privilege)
        .then((user) => {
            dispatch({
                type: ACTIONS.USERS_UPDATE_PROJECT_ACCESS,
                params: {
                    ...user,
                    project_uid: projectUid,
                    isCurrentProject: selectedProject ? selectedProject.uid === projectUid : false,
                },
            });
        })
        .catch((err) => {
            Promise.resolve(1);
            errorHandler(err, dispatch);
        });
}

export const updateUserAccess = (projectUid, companyUid, userUid, userType) => (dispatch, getState) => {
    const selectedProject = getState().adminPage.projectSelect;
    const privilege = getPrivilegeFromUserType(userType);
    updateUserAccessInProject({
        companyUid,
        projectUid,
        userUid,
        privilege,
        selectedProject,
        dispatch,
        getState,
    });
};


export const deleteUser = user => async (dispatch, getState) => {
    try {
        if (user.uid) {
            await api.deleteUser(user.uid);
        } else {
            for (let i = 0; i < user.length; i += 1) {
                // eslint-disable-next-line no-await-in-loop
                await api.deleteUser(user[i].uid);
            }
        }
        if (getState().worker.isWebSocketOn) {
            dispatch({
                type: ACTIONS.USERS_DELETE,
                params: user,
            });
        }
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

const createUserDetails = params => ({
    phone_number: params.phone_number ? params.phone_number : '',
});

const getProjectUid = user => user.project_uid || (user.project && user.project.uid);
const getPrivilege = user => user.project && user.project.priv;

export const editUser = user => async (dispatch, getState) => {
    try {
        const selectedProject = getState().adminPage.projectSelect;
        // console.log(`check   params: `, params);
        const userDetails = createUserDetails(user);
        const { uid } = user;
        const { appContext: { company } } = getState();

        const companyUid = company ? company.uid : storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID);

        let projectUid;
        let privilege = getPrivilegeFromUserType(user.userType);
        let adminStatus = 0;
        if (user.userType === USER_ROLES.COMPANY_ADMIN) {
            adminStatus = 1;
        } else {
            projectUid = getProjectUid(user);
            if (!privilege) {
                privilege = getPrivilege(user);
            }
        }

        const userToUpdate = {
            user: {
                email: user.email,
                first_name: user.first_name,
                last_name: user.last_name,
                user_details: userDetails,
                admin: adminStatus,
            },
        };
        // console.log(`check   user: `, user);
        const userData = await api.updateUser(userToUpdate, uid);
        const stateUser = getState().user;
        if (isUserDefaultAdmin(stateUser)
            && stateUser.onBehalf && stateUser.onBehalf.labUserData
            && stateUser.onBehalf.labUserData.email === userData.email
        ) {
            dispatch(updateLabUserData(userData));
            return;
        }

        if (user.currentUser) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
            dispatch({
                type: ACTIONS.USER_LOGIN,
                params: userData,
            });
            api.setParamsToHeader({ email: userData.email }, storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));
        } else if (getState().worker.isWebSocketOn) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
        }

        if (adminStatus === 0 && companyUid) {
            await updateUserAccessInProject({
                companyUid,
                projectUid,
                userUid: userData.uid,
                privilege,
                selectedProject,
                dispatch,
                getState,
            });
        }
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

export const createUser = params => (dispatch, getState) => {
    const selectedProject = getState().adminPage.projectSelect;
    const userDetails = {
        phone_number: params.phone_number ? params.phone_number : '',
    };
    const { appContext: { company } } = getState();

    const companyUid = company ? company.uid : storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID);

    const projectUid = params.project_uid;
    const privilege = getPrivilegeFromUserType(params.userType);
    let adminStatus = 0;
    if (params.userType === USER_ROLES.COMPANY_ADMIN) {
        adminStatus = 1;
    }
    const user = {
        user: {
            email: params.email,
            password: params.password,
            first_name: params.first_name,
            last_name: params.last_name,
            admin: adminStatus,
            send_notification: !!params.send_notification,
            user_details: userDetails,
        },
    };
    api.createUser(user)
        .then(async (userData) => {
            dispatch({
                type: ACTIONS.USERS_CREATE,
                params: userData,
            });

            if (adminStatus === 0 && companyUid) {
                await updateUserAccessInProject({
                    companyUid,
                    projectUid,
                    userUid: userData.uid,
                    privilege,
                    selectedProject,
                    dispatch,
                    getState,
                });
            }
        })
        .catch((err) => {
            Promise.resolve(1);
            console.error(err);
            errorHandler(err, dispatch);
        });
};

export const restoreUserPassword = email => (dispatch) => {
    dispatch({
        type: MODAL_ACTIONS.MODAL_SEND_NOTIFICATION_USER_EDIT,
    });

    return new Promise((resolve, reject) => {
        api.restorePassword(email)
            .then((user) => {
                dispatch({
                    type: MODAL_ACTIONS.MODAL_GET_NOTIFICATION_USER_EDIT,
                });

                // return user;
                resolve(user);
            }, error => reject(error))
            .catch((err) => {
                resolve();
                console.error(err);
                errorHandler(err, dispatch);
            });
    });
};
export const createUserSystemScope = params => (dispatch, getState) => {
    const userDetails = {
        phone_number: params.phone_number ? params.phone_number : '',
    };
    const user = {
        user: {
            email: params.email,
            password: params.password,
            first_name: params.first_name,
            last_name: params.last_name,
            user_details: userDetails,
            send_notification: !!params.send_notification,
            admin: 1,
        },
    };
    api.createUser(user)
        .then((userData) => {
            dispatch({
                type: ACTIONS.USERS_CREATE,
                params: userData,
            });

            return userData;
        })
        .catch((err) => {
            Promise.resolve(1);
            console.error(err);
            errorHandler(err, dispatch);
        });
};

export const editUserSystemScope = params => async (dispatch, getState) => {
    try {
        const userDetails = {
            phone_number: params.phone_number ? params.phone_number : '',
        };
        const { uid } = params;
        const user = {
            user: {
                email: params.email,
                first_name: params.first_name,
                last_name: params.last_name,
                user_details: userDetails,
            },
        };
        const userData = await api.updateUser(user, uid);
        if (params.currentUser) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
            dispatch({
                type: ACTIONS.USER_LOGIN,
                params: userData,
            });
            api.setParamsToHeader({ email: userData.email }, storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));
        } else if (getState().worker.isWebSocketOn) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
        }
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

export const changePassword = params => async (dispatch) => {
    try {
        await api.changePassword(params);
        storage.LocationStoreMethods.removeLocalStorageKeyValue(STORAGE_KEY.FORCE_TO_CHANGE_PASSWORD, params.decodedToken.user.email, null);
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};
