import * as api from 'api';
import { push } from 'connected-react-router';
import { ACTIONS, STORAGE_KEY } from 'common/constants';
import { NAME_OF_GEAR_BUTTONS } from 'common/ui-labels';
import { runWorker } from 'stores/actions/workerActions';
import { openChangePassword, openEditProfile } from 'stores/actions/modalActions/authActions';
import { openCompanyEditModalCompanyScope } from 'stores/actions/modalActions/companyModalActions';
import { setRolesToUser, userHasRole, userHasRoleCompanyAdmin, isCurrentUserCompanyAdmin, decodeToken, isTemporaryPassword } from 'utils/utils';
import { getPagesForScope, getNavButtonsForScope } from 'utils/scopeUtils';
import { _getHostName, getLocalStorageValue, LocationStoreMethods, setLocalStorageKeyValue, setSessionStorageKeyValue, reloadBrowser } from 'utils/browserStorage';
import { makeCopy, isUserDefaultAdmin, isUserSuperUser } from 'utils/simpleUtils';
import { USER_ROLES } from 'common/user-roles';
import { setAppContext } from 'stores/actions/appContextActions';
import { errorHandler } from 'stores/actions/errorHandlerActions';
import { clearStore } from 'stores/actions/commonActions';


/**
 *
 * @param params
 * @returns {{
 *      userParams: {
 *          access,
 *          project,
 *          onBehalf,
 *          email: string,
 *          token
 *      },
 *      appContextParams: {
 *          project,
 *          company
 *      }
 *  }}
 */
export const fillHeader = (params = LocationStoreMethods.getParamsFromStorage()) => {
    const { email, token, access, onBehalf, project, company } = params;
    const userParams = { email, token, access, onBehalf, project };
    const appContextParams = { company, project };

    if (company && getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID) === company.uid) {
        delete appContextParams.company;
    }

    appContextParams.hostName = _getHostName(undefined);

    api.setParamsToHeader({ ...appContextParams, ...userParams }, getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));

    return {
        appContextParams,
        userParams,
    };
};

/**
 *
 * @params params {SettingParamsType}
 * @returns {Function}
 */
export const settingUp = params => async (dispatch) => {
    const { appContextParams, userParams } = fillHeader(params);

    dispatch(setAppContext(appContextParams));

    if (userParams.email && userParams.token) {
        dispatch(userLoginByToken(userParams));
    }
};

const getCurrentUserData = (state) => {
    const currentUser = state.user;
    
    if (isUserDefaultAdmin(currentUser)) {
        return currentUser.onBehalf.labUserData;
    }
    const { email } = currentUser;
    return Object.values(state.entities.users.byUid).find(user => user.email === email);
};

const saveUserInfoAndReloadContext = (params) => {
    const userInfo = params;
    if (!userInfo.access) {
        const roles = setRolesToUser({
            admin: userInfo.admin,
            priv: userInfo.project ? userInfo.project.priv : null,
            isCompanyAdmin: userInfo.isCompanyAdmin,
            isSystemUser: userInfo.isSystemUser,
        });

        const currentProject = LocationStoreMethods.getParamsFromStorage().project;
        const hasAccessToProject = roles.some(r => (r === USER_ROLES.PROJECT_SYSTEM_ADMIN
            || r === USER_ROLES.PROJECT_OPERATIONAL_ADMIN
            || r === USER_ROLES.PROJECT_USER
        ));
        const currentRole = (userInfo.isCompanyAdmin && !!currentProject && hasAccessToProject)
            ? USER_ROLES.PROJECT_SYSTEM_ADMIN : roles[0];

        userInfo.access = {
            roles,
            currentRole,
            email: userInfo.email,
            isCompanyAdmin: userInfo.isCompanyAdmin,
            isSystemUser: userInfo.isSystemUser,
            isHighChartsLicensed: userInfo.isHighChartsLicensed,
        };
    }

    if (userInfo.company) {
        userInfo.company = {
            ...userInfo.company,
            ...userInfo.company.user_params,
        };
    } else if (!userInfo.company && userInfo.company !== null) {
        userInfo.company = LocationStoreMethods.getParamsFromStorage().company;
    }

    if (userInfo.project) {
        userInfo.project = {
            ...userInfo.project,
            ...userInfo.project.user_params,
        };
    } else if (!userInfo.project && userInfo.project !== null) {
        userInfo.project = LocationStoreMethods.getParamsFromStorage().project;
    }

    LocationStoreMethods.setParamsToStorage(userInfo);

    return userInfo;
};

export const userLogout = currentRole => (dispatch) => {
    const { email, onBehalf } = LocationStoreMethods.getParamsFromStorage();
    if (onBehalf) {
        api.logout({ email: onBehalf.email });
    } else {
        api.logout({ email });
    }
    
    LocationStoreMethods.clearUserParamsFromStorage(currentRole);
    fillHeader();

    dispatch(clearStore());

    dispatch({
        type: ACTIONS.USER_LOGOUT,
    });

    dispatch(push('/'));

    reloadBrowser();
};

export const userLoginByPassword = params => async (dispatch, getState) => {
    dispatch(push('/')); // clear url if we already have some to avoid problems with components which render by this some url

    const data = await api.auth(params);
    const decodedToken = decodeToken(data.token);

    if (data.isSystemUser) {
        setLocalStorageKeyValue(STORAGE_KEY.SYSTEM_COMPANY_UID, data.company.uid);
        delete data.company;
    }

    const userInfo = saveUserInfoAndReloadContext(data);

    // need to show first modal(recommendation to change password and welcome)
    // after loading all data, this params removed after loadEntities
    getState().user.password = params.password;
    getState().user.sign_in_count = data.sign_in_count;

    return {
        user: {
            ...data,
            uid: decodedToken.user.uid,
            isTemporaryPassword: isTemporaryPassword(params.password),
        },
        userInfo,
    };
};

/**
 *
 * @param params.email
 * @param params.token
 * @param params.access
 * @param params.scope
 * @returns {Function}
 */
export const userLoginByToken = params => async (dispatch, getState) => {
    dispatch(push('/')); // clear url if we already have some to avoid problems with components which render by this some url

    params.scope = createScope(params.access, getState);
    const {
        email, token, access, scope, onBehalf,
    } = params;
    await dispatch({
        type: ACTIONS.USER_LOGIN,
        params: {
            email, token, access, scope, onBehalf,
        },
    });
    await dispatch(push('/loading'));
    const onBehalfToken = onBehalf ? onBehalf.token : null;
    try {
        const { status, needUpdatePassword } = await api.validateToken({ email, token, onBehalfToken });
        if (status) {
            dispatch(runWorker());

            if (needUpdatePassword && !isUserSuperUser(params)) {
                const decodedToken = decodeToken(onBehalfToken || token);
                dispatch(openChangePassword({
                    uid: decodedToken.user.uid,
                    email,
                    token: onBehalfToken || token,
                    force: true,
                }));
            }
        } else {
            dispatch(userLogout());
        }
    } catch (err) {
        errorHandler(err, dispatch);
    }
};

/**
 *
 * @param access
 * @param getState
 * @returns Array{{
 *     label {string} intl
 *     title {string}
 *     iconClass {string}
 *     submenu {object}
 *     submenu.label {string} intl
 *     submenu.iconClass {string}
 *     submenu.subIconClass {string}
 *     submenu.onClick {function}
 * }}
 */
const getUserMenuItemsForScope = (access, getState) => (currentUserData) => {
    const { appContext: { company, helperLink } } = getState();
    const userMenuItems = [];

    if (company && userHasRoleCompanyAdmin(access)) {
        const submenu = [{
            label: NAME_OF_GEAR_BUTTONS.EDIT_COMPANY,
            iconClass: 'fas fa-building',
            subIconClass: 'fas fa-pencil-alt',
            dataId: 'user_menu_company_edit_company',
            onClick: () => openCompanyEditModalCompanyScope(company),
        }];

        if (!isCurrentUserCompanyAdmin(access.currentRole)) {
            submenu.push({
                label: NAME_OF_GEAR_BUTTONS.RETURN_TO_COMPANY_PAGE,
                iconClass: 'fas fa-directions',
                dataId: 'user_menu_company_submenu_back',
                onClick: userLogoutFromProjectSystemAdmin,
            });
        }

        userMenuItems.push({
            label: NAME_OF_GEAR_BUTTONS.COMPANY,
            dataId: 'user_menu_company',
            title: NAME_OF_GEAR_BUTTONS.COMPANY_TITLE,
            iconClass: 'fas fa-building',
            submenu,
        });
    }

    const currentUser = getCurrentUserData(getState());

    if (currentUser) {
        const userNameTemplate = user => `${user.first_name} ${user.last_name}`;
        const userFullName = (currentUser && userNameTemplate(currentUser)) || NAME_OF_GEAR_BUTTONS.USER_PROFILE;

        userMenuItems.push({
            label: userFullName,
            title: userFullName,
            iconClass: 'fas fa-user',
            dataId: 'user_menu_user',
            submenu: [
                {
                    label: NAME_OF_GEAR_BUTTONS.EDIT_PROFILE,
                    iconClass: 'fas fa-user-edit',
                    dataId: 'user_menu_user_submenu_edit_profile',
                    onClick: () => openEditProfile(currentUserData),
                },
                {
                    label: NAME_OF_GEAR_BUTTONS.SIGN_OUT,
                    iconClass: 'fas fa-sign-out-alt',
                    dataId: 'user_menu_user_submenu_sign_out',
                    onClick: () => userLogout(access.currentRole),
                },
            ],
        });
    }

    userMenuItems.push({
        label: NAME_OF_GEAR_BUTTONS.HELP_BUTTON,
        title: NAME_OF_GEAR_BUTTONS.HELP_BUTTON_TITLE,
        iconClass: 'far fa-question-circle',
        dataId: 'user_menu_help',
        submenu: [
            {
                label: NAME_OF_GEAR_BUTTONS.HELP_USER_GUIDE_PDF,
                iconClass: 'far fa-file-alt',
                dataId: 'user_menu_help_submenu_pdf',
                onClick: () => window.open(helperLink, '_blank'),
            },
        ],
    });

    return userMenuItems;
};

const createScope = (access, getState) => ({
    pages: getPagesForScope(access),
    navButtons: getNavButtonsForScope(access),
    userMenu: getUserMenuItemsForScope(access, getState),
});

export const updateLabUserData = params => (dispatch, getState) => {
    dispatch({
        type: ACTIONS.UPDATE_CURRENT_USER,
        params: {
            onBehalf: { ...getState().user.onBehalf, labUserData: params },
        },
    });
};

/**
 *
 * @param params
 * @param getState
 * @param labUserData
 * @returns {{
 *      email,
 *      token,
 *      company,
 *      isSuperAdmin,
 *      isSystemUser,
 *      isCompanyAdmin,
 *      sign_in_count,
 *      isHighChartsLicensed,
 *      needUpdatePassword,
 *      onBehalf: {
 *          logged,
 *          email,
 *          token,
 *          access: {
 *              roles string[],
 *              currentRole,
 *              email,
 *              isCompanyAdmin,
 *              isSystemUser,
 *              isHighChartsLicensed
 *          }
 *      }
 * }}
 */
const getCompanyOnBehalfUser = (params, getState, labUserData) => {
    const data = {
        ...params,
        onBehalf: {
            ...getState().user,
            labUserData,
        },
    };
    delete data.onBehalf.company;
    return data;
};

/**
 * Logging system user create by script in company scope using onBehalf mechanism
 * @param company
 * @param company.uid
 * @param company.name
 * @param company.projects
 * @param company.projects[].id
 * @param company.projects[].name
 * @param company.projects[].taggings_count
 * @param company.projects[].user_params
 * @param company.projects[].user_params.timezone
 * @param company.projects[].user_params.offset
 * @param company.projects[].user_params.manager
 * @param company.projects[].uid
 * @param company.projects[].scope
 * @param company.projects[].role_uid
 * @param company.projects[].company_uid
 * @param company.projects[].privileges
 * @param company.projects[].company_name
 * @param company.projects[].company_role_uid
 * @param company.user_params
 * @param company.user_params.tag_allocation
 * @param company.user_params.color
 * @param company.users
 * @param company.users[].uid
 * @param company.users[].admin
 * @param company.users[].email
 * @param company.users[].first_name
 * @param company.users[].last_name
 * @param company.users[].created_at
 * @param company.users[].updated_at
 * @param company.color
 * @param company.address
 * @param company.tag_allocation
 * @returns {void}
 */
export const userLoginAsCompanyAdmin = company => (dispatch, getState) => { // change context
    const labUserData = getCurrentUserData(getState());

    api.getCompanyAccess(company.uid)
        .then((params) => {
            const onBehalfUser = getCompanyOnBehalfUser(params, getState, labUserData);
            const userInfo = saveUserInfoAndReloadContext(onBehalfUser);
            dispatch(settingUp(userInfo));
        })
        .catch((err) => { errorHandler(err, dispatch); });
};

export const userLogoutFromCompanyAdmin = () => (dispatch, getState) => {
    delete getState().user.onBehalf.labUserData;
    const onBehalf = makeCopy(getState().user.onBehalf);

    setSessionStorageKeyValue(STORAGE_KEY.CURRENT_COMPANY, null);

    dispatch(clearStore());

    const userInfo = saveUserInfoAndReloadContext(onBehalf);
    dispatch(settingUp(userInfo));
};


export const userLoginAsProjectSystemAdmin = project => (dispatch, getState) => {
    const currentUser = makeCopy(getState().user);

    if (!userHasRole(currentUser.access, USER_ROLES.PROJECT_SYSTEM_ADMIN)) {
        return;
    }

    currentUser.access.currentRole = USER_ROLES.PROJECT_SYSTEM_ADMIN;
    currentUser.project = project;

    const userInfo = saveUserInfoAndReloadContext(currentUser);
    dispatch(settingUp(userInfo));
};

export const userLogoutFromProjectSystemAdmin = () => (dispatch, getState) => {
    const currentUser = makeCopy(getState().user);

    if (!userHasRole(currentUser.access, USER_ROLES.COMPANY_ADMIN)) {
        return;
    }

    currentUser.access.currentRole = USER_ROLES.COMPANY_ADMIN;
    currentUser.project = null;

    dispatch(clearStore());

    setSessionStorageKeyValue(STORAGE_KEY.CURRENT_PROJECT, null);

    const userInfo = saveUserInfoAndReloadContext(currentUser);
    dispatch(settingUp(userInfo));
};
