import { openAuditDownloadModal } from 'stores/actions/modalActions/commonModalActions';
import { getLocalStorageValue } from 'utils/browserStorage';
import { CUSTOM_LOCAL_STORE_KEYS, ROUTES, ADMIN_TABS_ROUTES } from 'common/constants';
import { NAME_OF_NAV_BUTTONS, NAMES_OF_TABS } from 'common/ui-labels';
import { ADMIN_PAGES, REPORT_PAGES, SYSTEM_PAGES, SYSTEM_BUTTONS, WIDGETS, LOGGER_PAGE_TABS } from 'common/pages';
import { USER_ROLES } from 'common/user-roles';
import { USER_SCOPE_KEYS, getCurrentUserScope } from 'common/scopes';
import { stringToBoolean, userHasAccessForRole } from 'utils/utils';
import { addModulesPageForUser, addModulesNavForUser } from 'utils/scopeAddons';

const { modules_list } = require('../../../addons/addonsList');

export const getDashboardWidgets = (access) => {
    const widgets = [WIDGETS.CURRENT_SHIFT_WIDGET, WIDGETS.PERSON_ACTIVITY_WIDGET, WIDGETS.CURRENT_ALARMS_WIDGET];
    
    if (userHasAccessForRole(access, USER_ROLES.PROJECT_SYSTEM_ADMIN)) {
        widgets.push(WIDGETS.WIN_STATUS_WIDGET);
    }
    return widgets;
};

/**
 *
 * @param access
 * @returns Array{{
 *     path {string} - '/admin'
 *     tabs: {Array}
 *     tabs.name {string} - intl("PROJECTS")
 *     tabs.systemName {string} - 'PROJECTS'
 *     tabs.path {string} - '/admin/nodes'
 * }}
 */
export const getPagesForScope = (access) => {
    const pages = [];
    const { currentRole } = access;
    const currentScope = getCurrentUserScope(currentRole);
    switch (currentScope) {
        case USER_SCOPE_KEYS.SYSTEM_SCOPE:
            pages.push({
                path: ROUTES.ADMIN_ROUTE,
                tabs: [ADMIN_PAGES.COMPANIES, ADMIN_PAGES.SUPER_ADMIN_USERS],
            });
            break;
        case USER_SCOPE_KEYS.COMPANY_SCOPE:
            pages.push({
                path: ROUTES.ADMIN_ROUTE,
                tabs: [ADMIN_PAGES.PROJECTS, ADMIN_PAGES.NODES],
            });
            break;
        case USER_SCOPE_KEYS.PROJECT_SCOPE:
        default:
            switch (currentRole) {
                case USER_ROLES.PROJECT_SYSTEM_ADMIN: {
                    pages.push({
                        path: ROUTES.ADMIN_ROUTE,
                        tabs: [ADMIN_PAGES.NODES],
                    });

                    pages.push({
                        path: ROUTES.SYSTEM_ROUTE,
                        tabs: [SYSTEM_PAGES.SERVER, SYSTEM_PAGES.ANCHORS],
                    });
                }
                case USER_ROLES.PROJECT_OPERATIONAL_ADMIN: {
                    const tabsForProjectAdmin = [
                        ADMIN_PAGES.USERS_PROJECT_SCOPE,
                        ADMIN_PAGES.TRACKABLE_OBJECTS,
                        ADMIN_PAGES.GROUPS,
                        ADMIN_PAGES.ZONES,
                        ADMIN_PAGES.NOTIFICATIONS,
                        ADMIN_PAGES.SHIFTS,
                    ];

                    const adminRoute = pages.find(page => page.path === ROUTES.ADMIN_ROUTE);
                    if (adminRoute) {
                        adminRoute.tabs.unshift(...tabsForProjectAdmin);
                    } else {
                        pages.push({
                            path: ROUTES.ADMIN_ROUTE,
                            tabs: [...tabsForProjectAdmin],
                        });
                    }
                }
                case USER_ROLES.PROJECT_USER: {
                    if (access.isHighChartsLicensed) {
                        pages.push(
                            { path: ROUTES.SITE_ROUTE },
                            { path: ROUTES.DASHBOARD_ROUTE },
                            {
                                path: ROUTES.REPORTS_ROUTE,
                                tabs: Object.values(REPORT_PAGES),
                            },
                        );
                    } else {
                        pages.push({ path: ROUTES.SITE_ROUTE }, { path: ROUTES.DASHBOARD_ROUTE });
                    }
                    break;
                }
                default:
            }
    }
    
    if (stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SYSTEM_LOGGER))) {
        pages.push({
            path: ROUTES.LOGGER_ROUTE,
            tabs: [
                LOGGER_PAGE_TABS.MESSAGE_PROCESSOR,
                LOGGER_PAGE_TABS.API,
                LOGGER_PAGE_TABS.LEVELS,
                LOGGER_PAGE_TABS.LOGS,
                LOGGER_PAGE_TABS.ACTIVITIES,
                LOGGER_PAGE_TABS.DEBUG,
            ],
        });
    }
    // This function modify pages;
    addModulesPageForUser(access.currentRole || access, pages, modules_list);
    
    return pages;
};

export const getButtonsForScope = (access) => {
    const buttons = [];
    const { currentRole } = access;
    const currentScope = getCurrentUserScope(currentRole);
    
    switch (currentScope) {
        case USER_SCOPE_KEYS.PROJECT_SCOPE:
        default:
            switch (currentRole) {
                case USER_ROLES.PROJECT_SYSTEM_ADMIN: {
                    buttons.push({
                        path: ROUTES.SYSTEM_ROUTE,
                        buttons: [SYSTEM_BUTTONS.SITE_MONITOR],
                    });
                }
                default:
            }
    }
    
    return buttons;
};

const getSubPagesForRoute = (pages, route) => pages.find(page => page.path === route)
    .tabs
    .map(page => ({
        label: page.name,
        dataCy: page.dataCy,
        to: page.path,
        iconClass: page.iconClass,
    }));

const getSubButtonsForRoute = (scopeButtons, route) => scopeButtons.find(button => button.path === route).buttons;

/**
 *
 * @param access
 * @returns Array{{
 *     label {string} intl string
 *     to {string?}
 *     iconClass {string}
 * }}
 */
export const getNavButtonsForScope = (access) => {
    const { currentRole } = access;
    const navButtons = [];
    const navPages = getPagesForScope(access);
    const scopeButtons = getButtonsForScope(access);

    const currentScope = getCurrentUserScope(currentRole);

    switch (currentScope) {
        case USER_SCOPE_KEYS.SYSTEM_SCOPE:
            navButtons.push(
                {
                    label: NAMES_OF_TABS.COMPANIES,
                    title: NAMES_OF_TABS.COMPANIES_TITLE,
                    to: ADMIN_TABS_ROUTES.COMPANIES,
                    dataCy: 'nav_menu_company',
                    iconClass: 'fas fa-building',
                },
                {
                    label: NAMES_OF_TABS.SUPER_ADMIN_USERS,
                    title: NAMES_OF_TABS.SUPER_ADMIN_USERS_TITLE,
                    to: ADMIN_TABS_ROUTES.SUPER_ADMIN_USERS,
                    dataCy: 'nav_menu_super_admin',
                    iconClass: 'fas fa-users',
                },
                {
                    label: NAMES_OF_TABS.AUDIT_DOWNLOAD,
                    title: NAMES_OF_TABS.AUDIT_DOWNLOAD_TITLE,
                    dispatchAction: openAuditDownloadModal,
                    dataCy: 'nav_menu_audit_download',
                    iconClass: 'fas fa-download',
                },
            );
            break;
        case USER_SCOPE_KEYS.COMPANY_SCOPE:
            navButtons.push(
                {
                    label: NAMES_OF_TABS.PROJECTS,
                    title: NAMES_OF_TABS.PROJECTS_TITLE,
                    to: ADMIN_TABS_ROUTES.PROJECTS,
                    dataCy: 'nav_menu_project',
                    iconClass: 'fas fa-project-diagram',
                },
                {
                    label: NAMES_OF_TABS.NODES,
                    title: NAMES_OF_TABS.NODES_TITLE,
                    to: ADMIN_TABS_ROUTES.NODES,
                    dataCy: 'nav_menu_nodes',
                    iconClass: 'fas fa-tags',
                },
                {
                    label: NAMES_OF_TABS.AUDIT_DOWNLOAD,
                    title: NAMES_OF_TABS.AUDIT_DOWNLOAD_TITLE,
                    dispatchAction: openAuditDownloadModal,
                    dataCy: 'nav_menu_audit_download',
                    iconClass: 'fas fa-download',
                },
            );
            break;
        case USER_SCOPE_KEYS.PROJECT_SCOPE:
        default:
            switch (currentRole) {
                case USER_ROLES.PROJECT_SYSTEM_ADMIN:
                    const systemSubmenuPages = getSubPagesForRoute(navPages, ROUTES.SYSTEM_ROUTE);
                    const systemSubmenuButtons = getSubButtonsForRoute(scopeButtons, ROUTES.SYSTEM_ROUTE);

                    navButtons.unshift({
                        label: NAME_OF_NAV_BUTTONS.SYSTEM,
                        title: NAME_OF_NAV_BUTTONS.SYSTEM_TITLE,
                        to: ROUTES.SYSTEM_ROUTE,
                        submenu: [...systemSubmenuPages, ...systemSubmenuButtons],
                        isButton: true,
                        dataCy: 'nav_menu_system',
                        iconClass: 'fas fa-server',
                    });
                case USER_ROLES.PROJECT_OPERATIONAL_ADMIN:
                    const adminMenuPages = getSubPagesForRoute(navPages, ROUTES.ADMIN_ROUTE);
                    navButtons.unshift({
                        label: NAME_OF_NAV_BUTTONS.ADMINISTRATIONS,
                        title: NAME_OF_NAV_BUTTONS.ADMINISTRATIONS_TITLE,
                        to: ROUTES.ADMIN_ROUTE,
                        submenu: adminMenuPages,
                        isButton: true,
                        dataCy: 'nav_menu_admin',
                        iconClass: 'fas fa-cogs',
                    });
                case USER_ROLES.PROJECT_USER:
                default:
                    if (access.isHighChartsLicensed) {
                        const reportsMenuPages = getSubPagesForRoute(navPages, ROUTES.REPORTS_ROUTE);

                        navButtons.unshift({
                            label: NAME_OF_NAV_BUTTONS.REPORTS,
                            title: NAME_OF_NAV_BUTTONS.REPORTS_TITLE,
                            to: ROUTES.REPORTS_ROUTE,
                            submenu: reportsMenuPages,
                            isButton: true,
                            dataCy: 'nav_menu_report',
                            iconClass: 'far fa-file-alt',
                        });
                    }
                    navButtons.unshift({
                        label: NAME_OF_NAV_BUTTONS.SITE,
                        title: NAME_OF_NAV_BUTTONS.SITE_TITLE,
                        to: ROUTES.SITE_ROUTE,
                        iconClass: 'fas fa-map-marked-alt',
                        dataCy: 'nav_menu_site',
                        expandable: true,
                    }, {
                        label: NAME_OF_NAV_BUTTONS.DASHBOARD,
                        title: NAME_OF_NAV_BUTTONS.DASHBOARD_TITLE,
                        to: ROUTES.DASHBOARD_ROUTE,
                        iconClass: 'fas fa-chart-bar',
                        dataCy: 'nav_menu_dashboard',
                        expandable: true,
                    });
            }
    }
    if (stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SYSTEM_LOGGER))) {
        navButtons.push({
            label: 'Logger',
            to: ROUTES.LOGGER_ROUTE,
            dataCy: 'nav_menu_logger',
            iconClass: 'fas fa-book-dead',
        });
    }
    // This function modify navButtons;
    addModulesNavForUser(access.currentRole || access, navButtons, modules_list);
    return navButtons;
};
