import hexToBinary from 'hex-to-binary';
import _Groupby from 'lodash/groupBy';
import _Get from 'lodash/get';
import jwt from 'jsonwebtoken';

import {
    DEFAULT_VALUES,
    TEMPORARY_PASSWORD_PREFIX, TIME, STORAGE_KEY, TRACKABLE_CATEGORY, FILTER,
    WIN_ANCHOR_TYPE, CLIENT_REPORT_TYPES, T2T_TYPE, TRACKABLE_TYPE, MAX_MAP_BOUNDS_LATLNG_OBJECT,
} from 'common/constants';
import { USER_ROLES } from 'common/user-roles';
import { currentVersion } from 'common/version';
import { SITE_VIEW_FILTER, COMPONENTS, LIVE_STATE, DB_NAMES } from 'common/ui-labels';
import { reloadBrowser, getLocalStorageValue, clearLocalStorage, setLocalStorageKeyValue, getSessionStorageValue, clearSessionStorage, LocationStoreMethods } from './browserStorage';
import { formatDbNameToHumanReadable } from './htmlUtils';
import { isCurrentShift } from './timeUtils';

/**
 *
 * @param token
 * @returns {{
 *     expiresIn,
 *     iat,
 *     user: {
 *         uid, role, email, authentication_token, admin, status, company_uid,
 *         projects[]: { project_uid, project_name, priv },
 *     }
 * }}
 */
export const decodeToken = (token = getSessionStorageValue(STORAGE_KEY.USER_TOKEN)) => jwt.decode(token);

export const findLocationIfExisted = (selectedLocation, locations) => {
    if (selectedLocation) {
        const location = locations.find(_location => _location.uid === selectedLocation.uid);
        if (location) {
            return location;
        }
    }
    return null;
};

export const getSublocation = (selectedSublocation, selectedLocation, sublocations, userEmail, projectUid) => {
    if (selectedLocation) {
        if (selectedSublocation) {
            const sublocation = sublocations.find(_sublocation => _sublocation.uid === selectedSublocation.uid);
            if (sublocation && sublocation.location_uid === selectedLocation.uid) {
                return sublocation;
            }
        }
        
        const firstSublocation = sublocations.find(_sublocation => _sublocation.location_uid === selectedLocation.uid);
        if (firstSublocation) {
            LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_SUBLOCATION, firstSublocation, userEmail, projectUid);
            return firstSublocation;
        }
    }
    return null;
};

export const getFloorplan = (selectedFloorplan, selectedSublocation, floorplans, userEmail, projectUid) => {
    if (selectedSublocation) {
        if (selectedFloorplan) {
            const floorplan = floorplans.find(_floorplan => _floorplan.uid === selectedFloorplan.uid);
            if (floorplan && floorplan.sublocation_uid === selectedSublocation.uid) {
                return floorplan;
            }
        }
        const firstFloorplan = floorplans.find(_floorplan => _floorplan.sublocation_uid === selectedSublocation.uid);
        if (firstFloorplan) {
            LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_FLOORPLAN, firstFloorplan, userEmail, projectUid);
            return firstFloorplan;
        }
    }
    return null;
};


export const findLocationBySublocation = (selectedLocation, selectedSublocation, locations, userEmail, projectUid) => {
    if (selectedSublocation) {
        if (selectedLocation) {
            const location = locations.find(_location => _location.uid === selectedLocation.uid);
            if (location && location.uid === selectedSublocation.location_uid) {
                return location;
            }
        }
        
        const foundLocation = locations.find(_loc => _loc.uid === selectedSublocation.location_uid);
        if (foundLocation) {
            LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_LOCATION, foundLocation, userEmail, projectUid);
            return foundLocation;
        }
    }
    return locations[0] || null;
};

export const findSublocationByFloorplan = (selectedSublocation, selectedFloorplan, sublocations, userEmail, projectUid) => {
    if (selectedFloorplan) {
        if (selectedSublocation) {
            const sublocation = sublocations.find(_sublocation => _sublocation.uid === selectedSublocation.uid);
            if (sublocation && sublocation.uid === selectedFloorplan.sublocation_uid) {
                return sublocation;
            }
        }
        
        const foundSublocation = sublocations.find(_sl => _sl.uid === selectedFloorplan.sublocation_uid);
        if (foundSublocation) {
            LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_SUBLOCATION, foundSublocation, userEmail, projectUid);
            return foundSublocation;
        }
    }
    return sublocations[0] || null;
};

export const findFloorplanBySelectedFloorplan = (selectedFloorplan, floorplans, userEmail, projectUid) => {
    if (selectedFloorplan) {
        const floorplan = floorplans.find(_floorplan => _floorplan.uid === selectedFloorplan.uid);
        if (floorplan) {
            return floorplan;
        }
    }
    if (floorplans[0]) {
        LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_FLOORPLAN, floorplans[0], userEmail, projectUid);
        return floorplans[0];
    }
    return null;
};

export const sortByField = (list, field) => {
    const sortedList = [...list];
    if (Array.isArray(list)) {
        return [...sortedList.sort((a, b) => (a[field] && a[field].localeCompare(b[field])))];
    }
    return list;
};

export const getCurrentFormattedVersion = () => {
    const { versionMajor, versionMiddle, versionMinor } = currentVersion();
    return [versionMajor, versionMiddle, versionMinor].join('.');
};

export const getCurrentVersion = () => {
    const { versionMajor, versionMiddle, versionMinor, date } = currentVersion();
    const splittedDate = date.split('.');
    const formattedDate = String((new Date(splittedDate[2], splittedDate[0] - 1, splittedDate[1])).getFullYear());
    
    return {
        rppVersion: [versionMajor, versionMiddle, versionMinor].join('.'),
        formattedDate,
    };
};

export const handlePackageVersion = () => {
    const packageVersion = getCurrentFormattedVersion();
    const prevPackageVersion = getLocalStorageValue(STORAGE_KEY.PACKAGE_VERSION);
    const betaPath = getLocalStorageValue(STORAGE_KEY.BETA_PATH);
    
    if (!prevPackageVersion || prevPackageVersion !== packageVersion) {
        clearLocalStorage();
        clearSessionStorage();
        setLocalStorageKeyValue(STORAGE_KEY.PACKAGE_VERSION, packageVersion);
        
        if (betaPath) {
            setLocalStorageKeyValue(STORAGE_KEY.BETA_PATH, betaPath);
        } else {
            reloadBrowser(); // In case of beta version the reload was made in BetaVersionButton
        }
    }
};

export const isTemporaryPassword = password => (password && !!~password.indexOf(TEMPORARY_PASSWORD_PREFIX) && password.split(TEMPORARY_PASSWORD_PREFIX)[1].length === 20);
export const isFirstUserLogin = params => (params && params.sign_in_count === 1);

export const setRolesToUser = (user) => {
    const roles = [];
    
    if (user.admin === USER_ROLES.COMPANY_ADMIN_STATUS) {
        roles.push(USER_ROLES.COMPANY_ADMIN);
    }
    
    switch (user.priv) {
        case undefined:
        case null:
            if (!user.isSystemUser && !user.isCompanyAdmin) {
                break;
            }
            if (user.isSystemUser) {
                roles.push(USER_ROLES.RPP_ADMIN);
            }
            if (user.isCompanyAdmin) {
                roles.push(USER_ROLES.COMPANY_ADMIN);
            }
        case USER_ROLES.PROJECT_SYSTEM_ADMIN_PRIVILEGE:
            roles.push(USER_ROLES.PROJECT_SYSTEM_ADMIN);
        case USER_ROLES.PROJECT_OPERATIONAL_ADMIN_PRIVILEGE:
            roles.push(USER_ROLES.PROJECT_OPERATIONAL_ADMIN);
        case USER_ROLES.PROJECT_USER_PRIVILEGE:
            roles.push(USER_ROLES.PROJECT_USER);
            break;
        case 0:
            roles.push(USER_ROLES.NO_ACCESS_USER);
        default:
            console.warn('No roles for this privilege');
    }
    return roles;
};

export const setUserType = (user) => {
    if (user.admin === USER_ROLES.COMPANY_ADMIN_STATUS) {
        return USER_ROLES.COMPANY_ADMIN;
    }
    const priv = _Get(user, 'project.priv', false);
    if (priv === USER_ROLES.PROJECT_SYSTEM_ADMIN_PRIVILEGE) {
        return USER_ROLES.PROJECT_SYSTEM_ADMIN;
    }
    if (priv === USER_ROLES.PROJECT_OPERATIONAL_ADMIN_PRIVILEGE) {
        return USER_ROLES.PROJECT_OPERATIONAL_ADMIN;
    }
    if (priv === USER_ROLES.PROJECT_USER_PRIVILEGE) {
        return USER_ROLES.PROJECT_USER;
    }
    return undefined;
};

export const userHasRole = (user, role) => {
    const userRoles = _Get(user, 'access.roles', _Get(user, 'roles', null));
    return (userRoles && userRoles.includes(role));
};

export const userHasRoleCompanyAdmin = user => (userHasRole(user, USER_ROLES.COMPANY_ADMIN));

export const isCurrentUserCompanyAdmin = userRole => (userRole === USER_ROLES.COMPANY_ADMIN);

export const userHasAccessForRole = (access, role) => access && (access.currentRole === role || access === role);

export const getPrivilegeFromUserType = (userType) => {
    if (userType === USER_ROLES.PROJECT_SYSTEM_ADMIN) {
        return USER_ROLES.PROJECT_SYSTEM_ADMIN_PRIVILEGE;
    }
    if (userType === USER_ROLES.PROJECT_OPERATIONAL_ADMIN) {
        return USER_ROLES.PROJECT_OPERATIONAL_ADMIN_PRIVILEGE;
    }
    if (userType === USER_ROLES.PROJECT_USER) {
        return USER_ROLES.PROJECT_USER_PRIVILEGE;
    }
    return 0;
};

export const userHasPrivileges = (currentUserType, comparedUserType) => {
    if (currentUserType === USER_ROLES.COMPANY_ADMIN) {
        return true;
    }
    return getPrivilegeFromUserType(currentUserType) >= getPrivilegeFromUserType(comparedUserType);
};

export function userInProjectScope(userRole) {
    const projectScope = [
        USER_ROLES.PROJECT_OPERATIONAL_ADMIN,
        USER_ROLES.PROJECT_SYSTEM_ADMIN,
        USER_ROLES.PROJECT_USER,
    ];
    return projectScope.indexOf(userRole) !== -1;
}

export function nodeFilterNotWin(nodes) {
    return nodes.filter(node => node.node_type && node.node_type.toLowerCase() !== 'win');
}

/**
 * Add to element value = uid and label = name. value/label using in React-select control
 * @param x
 * @returns x
 */
export const addValueLabelToValue = x => ({ ...x, value: x.uid, label: x.name });

export const formatToLabelValueList = (arr, labelFieldName = 'name') =>
    arr.map((el) => ({ label: el[labelFieldName], value: el.uid }));

export const filterSelectBoxOptions = (options, text, selectedItems) => {
    const isItemSelected = (itemUid) => selectedItems && selectedItems.some((item) => (item.value === itemUid));

    return options.filter((o) => (o.label && o.label.includes(text)) && !isItemSelected(o.value));
};
export const getInclusionsExclusionsObjects = (zone_exceptions = [], zone_exceptions_groups = [], trackables = [], groups = []) => {
    const trackables_and_groups = [...zone_exceptions, ...zone_exceptions_groups];
    
    return trackables_and_groups.map((entityUid) => {
        const trackable = trackables.find(tko => tko.uid === entityUid);
        const group = groups.find(gr => gr.uid === entityUid);
        if (trackable) {
            return addValueLabelToValue({ uid: trackable.uid, name: trackable.name });
        }
        if (group) {
            return addValueLabelToValue({ uid: group.uid, name: group.name });
        }
        return undefined;
    }).filter(x => x);
};

export function getNotificationTypes(notificationTypes, intl) {
    return notificationTypes
        .map(notificationName => ({
            uid: notificationName,
            name: formatDbNameToHumanReadable(intl, notificationName),
        }))
        .map(addValueLabelToValue);
}

export const filterTrackables = (trackables, selectedGroup) => {
    if (selectedGroup) {
        const { uid } = selectedGroup;
        return trackables.filter(trackable => trackable.groups && trackable.groups.some(group => group === uid));
    }
    return [];
};

export const filterTrackablesByGroups = (trackables, selectedGroups) => {
    if (selectedGroups.length > 0) {
        const uids = selectedGroups.map(x => x.uid);
        return trackables.filter(trackable => trackable.groups && trackable.groups.some(group => uids.some(uid => group === uid)));
    }
    return [];
};

export const isAnchor = type => type === WIN_ANCHOR_TYPE.ANCHOR;
export const isBridge = type => type === WIN_ANCHOR_TYPE.BRIDGE;
export const isAnchorOrBridge = type => isAnchor(type) || isBridge(type);

export const isAnchorLive = anchor => anchor.node && anchor.node.responsive;

export const hasAlarms = tko => tko && tko.alarms && tko.alarms.length > 0;

export const getTrackableStatus = (tko, intl) => {
    const liveState = tko.isResponsive() ? LIVE_STATE.LIVE(intl) : LIVE_STATE.NOT_RESPONDING(intl);
    return hasAlarms(tko) ? LIVE_STATE.ALARM(intl) : liveState;
};

export const getAdminPageSelectedGroups = (state) => {
    const { selectedItems } = state.adminPage;
    const groups = state.entities.groups.byUid;
    
    return selectedItems.filter(item => groups[item.uid]);
};

export const getAdminPageSelectedGroup = (state) => {
    const selectedGroups = getAdminPageSelectedGroups(state);
    return selectedGroups.length === 1 ? selectedGroups[0] : null;
};

export const userHasFullAccess = user => (userHasRole(user, USER_ROLES.PROJECT_OPERATIONAL_ADMIN));

export const stringToBoolean = (value) => {
    if (value === 'false' || value === false) {
        return false;
    } else if (value === 'true' || value === true) {
        return true;
    }
    return undefined;
};

export const printReportByTagID = (tagId, company, project, text, filterDescription) => {
    const content = document.getElementById(tagId);
    const pri = document.getElementById('ifmcontentstoprint').contentWindow;
    pri.document.open();
    pri.document.write(`<h1>${company}</h1>`);
    pri.document.write(`<h2>${project}</h2>`);
    pri.document.write(`<h3>${text}</h3>`);
    if (filterDescription) {
        Object.keys(filterDescription).forEach((key) => {
            pri.document.write(`<h3>${key}: ${filterDescription[key].join(', ')}</h3>`);
        });
    }
    pri.document.write(content.innerHTML);
    pri.document.close();
    pri.focus();
    pri.print();
};

export const getReportParams = (state, reportType) => {
    switch (reportType) {
        case CLIENT_REPORT_TYPES.ZONE_ACTIVITY_REPORT: {
            return state.reports.zoneActivityReport;
        }
        case CLIENT_REPORT_TYPES.TRACKABLE_DISTANCE_REPORT: {
            return state.reports.trackableDistanceReport;
        }
        case CLIENT_REPORT_TYPES.PWS_REPORT: {
            return state.reports.pwsReport;
        }
        case CLIENT_REPORT_TYPES.ALARMS_REPORT: {
            return state.reports.alarmsReport;
        }
        case CLIENT_REPORT_TYPES.ATTENDANCE: {
            return state.reports.attendanceReport;
        }
        default: {
            return {};
        }
    }
};

export const getReportFilterOptions = (state, reportType) => {
    const reportOptions = getReportParams(state, reportType);
    return reportOptions.filter || {};
};

export const separateLocationsAndSublocations = (loc_sub, storeLocations, storeSublocations) => {
    const locations = [];
    const sublocations = [];
    if (loc_sub) {
        loc_sub.forEach((uid) => {
            const location = storeLocations.find(_location => _location.uid === uid);
            const sublocation = storeSublocations.find(_sublocation => _sublocation.uid === uid);
            if (location) {
                locations.push(location.uid);
            }
            if (sublocation) {
                sublocations.push(sublocation.uid);
            }
        });
    }
    return { locations, sublocations };
};

export const isMacsEqual = (string_one, string_two) => {
    if ((typeof string_one === 'string') && (typeof string_two === 'string')) {
        return string_one.toLowerCase() === string_two.toLowerCase();
    }
    console.error('Function isMacsEqual(). Arguments should be string type', string_one, string_two);
    return undefined;
};

export const macToUpperCase = (mac_address) => {
    if (typeof mac_address === 'string') {
        return mac_address.toUpperCase();
    }
    console.warn('Function macToUpperCase(). MAC address is not a string');
    return mac_address;
};

export const valueNaN = (_value, _else) => {
    if (!isNaN(_value)) {
        return _value;
    }
    return _else;
};

const collisionAvoidanceFromConfig = (value) => {
    if (value.m) {
        // if (value.m === 'off') return ' ';
        if (value.m === 'a') return true;
        if (value.m === 'v') return false;
    }
    return false;
};

const collisionAvoidanceToConfig = (value) => {
    if (value) {
        return { m: 'a', d: 10000, a: 90 };
    }
    return { m: 'v' };
};

export const parseJnameValueToJson = (jname, value) => {
    if (jname === 'cav_cfg') {
        return collisionAvoidanceToConfig(value);
    }
    
    return value;
};

export const parseValueJnameToString = (jname, value) => {
    if (jname === 'cav_cfg') {
        return collisionAvoidanceFromConfig(value);
    }
    
    return value;
};

export const handleLeafletDeltaZoom = (leaflet) => {
    leaflet.leafletElement.scrollWheelZoom.__proto__._performZoom = function () {
        const map = this._map;
        const zoom = map.getZoom();
        
        map._stop(); // stop panning and fly animations if any
        
        // map the delta with a sigmoid function to -4..4 range leaning on -1..1
        const delta = map._limitZoom(zoom + (this._delta > 0 ? DEFAULT_VALUES.MAP_ZOOM.DELTA_SCROLL : -DEFAULT_VALUES.MAP_ZOOM.DELTA_SCROLL)) - zoom;
        
        this._delta = 0;
        this._startTime = null;
        
        if (!delta) {
            return;
        }
        
        if (map.options.scrollWheelZoom === 'center') {
            map.setZoom(zoom + delta);
        } else {
            map.setZoomAround(this._lastMousePos, zoom + delta);
        }
    };
    
    return leaflet;
};

export const isNodeUnresponsive = (node, interval = TIME.HEARTBEAT_UPDATE_INTERVAL) => {
    if (!node || !node.last_heard || Number.isNaN(interval)) return true;
    
    const COUNT_FOR_CHECK_LOSTING = 4;
    
    const timeLost = interval * COUNT_FOR_CHECK_LOSTING;
    const lastHeard = new Date(node.last_heard);
    
    const now = new Date();
    return (now - lastHeard) > timeLost;
};

/**
 *
 * @param value {string} 2 chars of HEX value '[0..F][0..F]'
 * @return {{bin, channels: *}}
 */
export function parseHexToChannelsArray(value) {
    const bin = hexToBinary(value); // Number.parseInt(hex, 2);
    // console.log(bin);
    const channels = bin.split('').reduceRight((r, x, i) => {
        if (Number.parseInt(x) === 1) {
            r.push(bin.length - i - 1);
        }
        return r;
    }, []);
    // console.log(typeof(bin));
    return { bin, channels };
}

export function parseChannelsArrayToHex(array, bits) {
    // console.log(array, bits);
    const res = array.reduceRight((r, x) => {
        r[bits - x - 1] = 1;
        // console.log(r, x, i);
        return r;
    }, Array(bits).fill(0)).join('');
    // console.log(res);
    
    return parseInt(res, 2).toString(16).toUpperCase();
}

export function parseHexToChannels(config) {
    // from hex fo numbers
    // console.log('Start config', config);
    
    const numberedChannels = config.reduce((r, x) => {
        const { channels } = parseHexToChannelsArray(x.channels);
        // console.log('channels', x.channels, channels);
        channels.forEach((c) => {
            const copy = JSON.parse(JSON.stringify(x));
            r.push(Object.assign(copy, { channels: c }));
        });
        return r;
    }, []);
    
    return numberedChannels;
}

export function parseChannelsToHex(config) {
    /*
    const values = [
        { type: 'AI', channels: 0, op_mode: 'periodic', interval: 6, reliable: false },
        { type: 'AI', channels: 1, op_mode: 'periodic', interval: 6, reliable: false },
        { type: 'DI', channels: 2, op_mode: 'on-request', reliable: false },
        { type: 'DI', channels: 3, op_mode: 'on-request', reliable: false },
        { type: 'DO', channels: 4, enabled: true, reliable: false },
        { type: 'DO', channels: 5, enabled: true, reliable: false }
    ];
    */
    // from numbers to hex
    const groupedType = _Groupby(config, 'type');
    const backConfig = []; // 0, 1, 2, 3, 4, 5
    // console.log('groupedType 1', groupedType);
    Object.keys(groupedType).forEach((keyType) => {
        if ((keyType === 'AI' || keyType === 'DI') && groupedType[keyType].length > 0) {
            groupedType[keyType] = _Groupby(groupedType[keyType], 'op_mode');
            Object.keys(groupedType[keyType]).forEach((keyOpMode) => {
                const groupedChannel = groupedType[keyType][keyOpMode][0];
                groupedChannel.channels = parseChannelsArrayToHex(groupedType[keyType][keyOpMode].map(x => x.channels), 8);
                // backConfig[groupedChannel.channels] = groupedChannel;
                backConfig.push(groupedChannel);
            });
        }
        if (keyType === 'DO' && groupedType[keyType].length > 0) {
            groupedType[keyType] = _Groupby(groupedType[keyType], 'enabled');
            Object.keys(groupedType[keyType]).forEach((keyEnabled) => {
                const groupedChannel = groupedType[keyType][keyEnabled][0];
                groupedChannel.channels = parseChannelsArrayToHex(groupedType[keyType][keyEnabled].map(x => x.channels), 8);
                // backConfig[groupedChannel.channels] = groupedChannel;
                backConfig.push(groupedChannel);
            });
        }
    });
    return backConfig;
}

export const channelList = [0, 1, 2, 3, 4, 5]; // , 6, 7
export const sensorSpecialCasesListQ2 = ['UART_0'];

const setOpMode = (enabled, interval) => {
    const num = Number.parseInt(interval);
    if (!isNaN(num) && num > 0) {
        return 'periodic';
    }
    return enabled ? 'on-request' : 'disabled';
};

export const prepareSensorInterfaceForServer = sensorInterface => (
    sensorInterface ? parseChannelsToHex(sensorInterface)
        .filter(x => x.type !== FILTER.NA)
        .map((x) => {
            if (x.type === 'DI' || x.type === 'AI') {
                return {
                    type: x.type,
                    channels: x.channels.length < 2 ? `0${x.channels}` : x.channels,
                    op_mode: setOpMode(x.enabled, x.interval),
                    interval: x.interval,
                };
            } else if (x.type === 'DO') {
                return {
                    type: x.type,
                    channels: x.channels.length < 2 ? `0${x.channels}` : x.channels,
                    enabled: !!x.enabled,
                    default_level: 0,
                };
            }
            return undefined;
        }) : []
);

export const prepareSensorSpecialCasesForServer = sensorSpecialCases => (
    sensorSpecialCases && Object.keys(sensorSpecialCases).length > 0 ? Object.keys(sensorSpecialCases)
        .map((key) => {
            if (key === 'UART_0') {
                return {
                    type: key,
                    enabled: sensorSpecialCases[key],
                    reliable: true,
                };
            } else if (key === 'ALM_IN') {
                return {
                    type: key,
                    enabled: sensorSpecialCases[key],
                };
            } else if (key === 'ALM_OUT') {
                return {
                    type: key,
                    enabled: sensorSpecialCases[key],
                };
            }
            return {
                type: key,
                enabled: true,
            };
        }) : []
);

/**
 *
 * @param uids {array} array of group/trackable uids
 * @param groups {array<object>}
 * @param trackables {array<object>}
 * @return {string}
 */
export const getTrackableGroupNameByUid = (uids, groups, trackables) => {
    const result = [];
    uids.forEach((uid) => {
        const group = groups.find(x => x.uid === uid);
        if (group) {
            result.push(group.name);
        } else {
            const trackable = trackables.find(x => x.uid === uid);
            if (trackable) {
                result.push(trackable.name);
            }
        }
    });
    return result.join(', ');
};

export function isShowWin(currentRole) {
    return currentRole !== USER_ROLES.PROJECT_USER && currentRole !== USER_ROLES.PROJECT_OPERATIONAL_ADMIN;
}

export const getAnchorType = (intl, anchor) => {
    if (anchor && anchor.type) {
        switch (anchor.type) {
            case WIN_ANCHOR_TYPE.BRIDGE:
                return DB_NAMES.BRIDGE(intl);
            case WIN_ANCHOR_TYPE.ANCHOR:
                return DB_NAMES.ANCHOR(intl);
            default:
                return anchor.type;
        }
    }
    return DB_NAMES.ANCHOR(intl);
};

export const isStationary = trackable => _Get(trackable, 'tag_config.stationary', false);

export const getTrackableType = (intl, trackable) => {
    if (isStationary(trackable)) {
        return SITE_VIEW_FILTER.TITLES.STATIONARY(intl);
    }

    if (trackable && trackable.category) {
        switch (trackable.category) {
            case TRACKABLE_CATEGORY.VEHICLE:
                return DB_NAMES.VEHICLE(intl);
            case TRACKABLE_CATEGORY.PERSON:
                return DB_NAMES.PERSON(intl);
            case TRACKABLE_CATEGORY.ASSET:
                return DB_NAMES.ASSET(intl);
            case TRACKABLE_CATEGORY.UNSPECIFIED: {
                if (trackable && trackable.type) {
                    switch (trackable.type) {
                        case TRACKABLE_TYPE.NAV:
                            return DB_NAMES.NAVIGATION(intl);
                        case TRACKABLE_TYPE.MOBILE:
                            return DB_NAMES.MOBILE(intl);
                        default:
                            return trackable.type;
                    }
                } else {
                    return DB_NAMES.UNSPECIFIED(intl);
                }
            }
            default:
                return trackable.category;
        }
    }
    return COMPONENTS.FIELDS.NODE.TRACKABLE_FUNC(intl);
};

export const getNodeByMacAddress = (store, mac_address) => {
    if (mac_address) {
        return store.getState().entities.nodes.byMac[mac_address.toUpperCase()];
    }
    return null;
};

export function getNodeByMacFromMap(nodesMap, mac_address) {
    return nodesMap[mac_address && mac_address.toUpperCase()];
}

export const isAssetTag = category => category === TRACKABLE_CATEGORY.ASSET;
export const isUnspecified = category => category === TRACKABLE_CATEGORY.UNSPECIFIED;
export const isVehicle = category => category === TRACKABLE_CATEGORY.VEHICLE;
export const isPerson = category => category === TRACKABLE_CATEGORY.PERSON;
export const isRoleMonitor = role => role === T2T_TYPE.MONITOR;
export const isRoleAnnouncer = role => role === T2T_TYPE.ANNOUNCER;

/**
 *
 * @param role see T2T_TYPE
 * @param category see TRACKABLE_CATEGORY
 * @returns {{
 *      showPin: boolean
 *      showAngle: boolean
 *      showAlertDistance: boolean
 *      showSafeDistance: boolean
 *      showsT2tNoAlarmUid: boolean
 *      showDistance: boolean
 *      showDelay: boolean
 *      showAlarmBack: boolean
 * }}
 */
export const showPwsFields = (role, category) => {
    const roleAnnouncer = isRoleAnnouncer(role);
    const roleMonitor = isRoleMonitor(role);
    const categoryVehicle = isVehicle(category);
    const categoryPerson = isPerson(category);
    return {
        showDistance: roleAnnouncer,
        showAlertDistance: roleAnnouncer,
        showSafeDistance: roleAnnouncer && categoryVehicle,
        showDelay: (roleMonitor && categoryPerson) || (roleAnnouncer && categoryPerson),
        showPin: roleMonitor || roleAnnouncer,
        showAngle: categoryVehicle && roleAnnouncer,
        showAlarmBack: roleMonitor,
        showsT2tNoAlarmUid: roleMonitor,
    };
};

/**
 * Uses in input type text for preventing any not number input
 * @param e {Event} onChange event
 * @returns {number}
 */
export function preventInputNotNumber(e) {
    const { value } = e.target;
    if (value === '-') {
        return value;
    }
    const inputNumber = Number(value);
    if (isNaN(inputNumber) || value.includes(' ')) {
        e.preventDefault();
        return undefined;
    }
    return inputNumber;
}

export const getDeviceTypeClass = (device) => {
    const deviceType = (device && (device.category || device.type)) || '';
    const stationary = (isStationary(device) && 'stationary') || '';
    return `${deviceType} ${stationary}`;
};

function trackableHasProperVersion(tko, node) {
    const hasRole = tko.hasLowLatencyConfigurations();
    return hasRole ? tko.hasProperVersionForLowLatencyLink(node) : !hasRole;
}

export function getLLLConfigFromFormValues({
    role, alarm_back, distance, angle, 
}) {
    const t2t_cfg = {};
    if (role === T2T_TYPE.MONITOR) {
        t2t_cfg.role = T2T_TYPE.MONITOR;
        t2t_cfg.alarm_back = alarm_back || false;
    } else if (role === T2T_TYPE.ANNOUNCER) {
        t2t_cfg.role = T2T_TYPE.ANNOUNCER;
        t2t_cfg.distance = distance || 0;
        t2t_cfg.angle = angle || 0;
    } else {
        t2t_cfg.role = '';
    }
    return t2t_cfg;
}

export function isOpenLllValidationModal({ trackable, node }) {
    const hasProperVersion = trackableHasProperVersion(trackable, node);
    
    return !hasProperVersion;
}

export function getTrackableLLLConfig(values, trackableObject) {
    const oldGroup = trackableObject[values.uid];
    return _Get(oldGroup, 'low_latency_link_config');
}

export function getLLLConfig(trackable) {
    return getLLLConfigFromFormValues(trackable);
}

export function getLength(value) {
    if (typeof value === 'string') {
        const zhRegex = /(?:[\u3400-\u4DBF\u4E00-\u9FFC\uFA0E\uFA0F\uFA11\uFA13\uFA14\uFA1F\uFA21\uFA23\uFA24\uFA27-\uFA29]|[\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879\uD880-\uD883][\uDC00-\uDFFF]|\uD869[\uDC00-\uDEDD\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD884[\uDC00-\uDF4A])/g;
        return value.match(zhRegex) ? value.match(zhRegex).length + value.replace(zhRegex, '').length : value.length;
    }
    return 0;
    // return (new TextEncoder('utf-8').encode(value)).length;
}

export function writeFiltersIntoPdf(filterDescription, doc, top = 117) {
    const keyCount = Object.keys(filterDescription).length;
    if (filterDescription) {
        Object.keys(filterDescription).forEach((key, i) => {
            doc.text(` ${key}: ${filterDescription[key].join(', ')}`, 72, ((i + 1) * 20) + top);
        });
    }
    return keyCount;
}


export function hasProperVersionForLowLatencyLink(node, low_latency_link_config) {
    if (!node) {
        return true;
    }
    const cbid = _Get(node, 'hw_info.cbid', null);

    const { role } = low_latency_link_config || {};

    if (role) {
        return cbid && cbid.toUpperCase() !== 'FFFF';
    }
    return true;
}

export function getMaxMapBoundsArray() {
    const maxMapBounds = Object.assign({}, MAX_MAP_BOUNDS_LATLNG_OBJECT);

    return Object.values(maxMapBounds).map(bounds => Object.values(bounds));
}

export async function sleep(ms = 1000) {
    return new Promise((resolve) => {
        setTimeout(() => {
            resolve();
        }, ms);
    });
}


export const getUID = () => {
    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1);
    }
    
    return s4() + s4() + s4() + s4() + s4() + s4().substr(0, 2);
};

export function getShifts(locations) {
    return locations.reduce((memo, location) => {
        memo.push(...location.shift);
        return memo;
    }, []);
}

export function roundTo(number, roundValue) {
    const round = 10 ** roundValue;
    return Math.round(number * round) / round;
}

export function getActiveShiftBySiteUid(selectedSiteUid, sites, timeZone) {
    const selectedSite = sites.find(s => s.uid === selectedSiteUid) || {};
    const shifts = [...selectedSite.shift];

    if (!Array.isArray(shifts) || shifts.length === 0) {
        return null;
    }

    return shifts.find(item => isCurrentShift(item, timeZone));
}

export function getClusterMarkers(cluster, markersArray = []) {
    markersArray.push(...cluster._markers);
    for (let i = cluster._childClusters.length - 1; i >= 0; i -= 1) {
        getClusterMarkers(cluster._childClusters[i], markersArray);
    }
    return markersArray;
}
